#
# AC_LIB_VLIB(MIN_VERSION)
# -----------
# Detect v-lib and set CXXFLAGS and LDFLAGS as needed
# Set variable vlib_static to point to the static lib
# Set use_static_vlib to 1 if a static link with vlib was requested
# Substitute LIBVLIB with: $vlib_static or -lvlib
AC_DEFUN([AC_LIB_VLIB],
	[
		AC_ARG_WITH(	vlib,
				[AC_HELP_STRING([--with-vlib=DIR],
					[where vlib is installed])	],
				[tmpVLIB="$with_vlib"	])

		AC_ARG_ENABLE(	static-vlib,
				[AC_HELP_STRING(	[--enable-static-vlib],
							[Use static version of vlib])],
				[if test "$enable_static_vlib" = "yes" ; then
					use_static_vlib=1
				else
					use_static_vlib=0
				fi],
				[use_static_vlib=0])

		AC_REQUIRE([AC_PROG_CXX])

		AC_REQUIRE([AC_LANG_CPLUSPLUS])

		AC_MSG_CHECKING([for V-lib])

		AC_MSG_RESULT([])

		AC_PATH_PROG(VLIB_CONFIG, v-config, -, $tmpVLIB/bin:$PATH)

		if test "$VLIB_CONFIG" = "-" ; then
			AC_MSG_ERROR([v-lib is required])
		else
			AC_CACHE_CHECK(	[v-lib version],
					[vlib_cv_lib_version],
					[
						vlib_cv_lib_version=`$VLIB_CONFIG --version`
					])
		fi

		tmpVERS="$vlib_cv_lib_version"

dnl		tmpVLIB_V1=`echo "$tmpVERS" | sed 's,\(\.[[0-9]]*\)\{3\}$,,1'`
dnl		tmpVLIB_V2=`echo "$tmpVERS" | sed 's,^\([[0-9]]*\.\)\{1\},,1' | sed 's,\(\.[[0-9]]*\)\{2\}$,,1'`
dnl		tmpVLIB_V3=`echo "$tmpVERS" | sed 's,^\([[0-9]]*\.\)\{2\},,1' | sed 's,\(\.[[0-9]]*\)\{1\}$,,1'`
dnl		tmpVLIB_V4=`echo "$tmpVERS" | sed 's,^\([[0-9]]*\.\)\{3\},,1'`

dnl		tmpVLIB_RV1=`echo "$1" | sed 's,\(\.[[0-9]]*\)\{3\}$,,1'`
dnl		tmpVLIB_RV2=`echo "$1" | sed 's,^\([[0-9]]*\.\)\{1\},,1' | sed 's,\(\.[[0-9]]*\)\{2\}$,,1'`
dnl		tmpVLIB_RV3=`echo "$1" | sed 's,^\([[0-9]]*\.\)\{2\},,1' | sed 's,\(\.[[0-9]]*\)\{1\}$,,1'`
dnl		tmpVLIB_RV4=`echo "$1" | sed 's,^\([[0-9]]*\.\)\{3\},,1'`

		tmpVLIB_V1=`echo "$tmpVERS" | cut -d'.' -f1`
		tmpVLIB_V2=`echo "$tmpVERS" | cut -d'.' -f2`
		tmpVLIB_V3=`echo "$tmpVERS" | cut -d'.' -f3`
		tmpVLIB_V4=`echo "$tmpVERS" | cut -d'.' -f4`

		tmpVLIB_RV1=`echo "$1" | cut -d'.' -f1`
		tmpVLIB_RV2=`echo "$1" | cut -d'.' -f2`
		tmpVLIB_RV3=`echo "$1" | cut -d'.' -f3`
		tmpVLIB_RV4=`echo "$1" | cut -d'.' -f4`

		BAD=0
		if test "$tmpVLIB_V1" -lt "$tmpVLIB_RV1" ; then
			BAD=1
		else
			if test "$tmpVLIB_V1" -eq "$tmpVLIB_RV1"  ; then
				if test "$tmpVLIB_V2" -lt "$tmpVLIB_RV2" ; then
					BAD=1
				else
					if test "$tmpVLIB_V2" -eq "$tmpVLIB_RV2" ; then
						if test "$tmpVLIB_V3" -lt "$tmpVLIB_RV3" ; then
							BAD=1
						else
							if test "$tmpVLIB_V3" -eq "$tmpVLIB_RV3" ; then
								if test "$tmpVLIB_V4" -lt "$tmpVLIB_RV4" ; then
									BAD=1
								fi
							fi
						fi
					fi
				fi
			fi
		fi

		if test "$BAD" -eq 1 ; then
			AC_MSG_ERROR([V-lib $tmpVLIB_RV1.$tmpVLIB_RV2.$tmpVLIB_RV3.$tmpVLIB_RV4 or newer is required])
		fi

dnl	vlib_cv_extraenv_CXXFLAGS

		AC_CACHE_CHECK(	[for required extra CXXFLAGS],
				[vlib_cv_extraenv_CXXFLAGS],
				[ AC_TRY_COMPILE(	[#include <v/mem/longline.h>	],
							[],
							[vlib_cv_extraenv_CXXFLAGS=none],
							[	tmpVLIB_xtra=`$VLIB_CONFIG --cxxflags`
								vlib_cv_extraenv_CXXFLAGS="$tmpVLIB_xtra"
								tmpCXXFLAGS="${CXXFLAGS}"
								CXXFLAGS="${CXXFLAGS} $tmpVLIB_xtra"

								AC_TRY_COMPILE(	[#include <v/mem/longline.h>    ],
										[],
										[vlib_cv_extraenv_CXXFLAGS="$tmpVLIB_xtra"],
										[vlib_cv_extraenv_CXXFLAGS=""]
									)

								CXXFLAGS="${tmpCXXFLAGS}"
								]
					)]
			)

		if test "$vlib_cv_extraenv_CXXFLAGS" = "" ; then
			AC_MSG_ERROR([Unable to compile sample program])
		fi

		if test ! "$vlib_cv_extraenv_CXXFLAGS" = "none" ; then
			CXXFLAGS="${CXXFLAGS} $vlib_cv_extraenv_CXXFLAGS"
		fi

		AC_CACHE_CHECK(	[for required extra LDFLAGS],
				[vlib_cv_extraenv_LDFLAGS],
				[
					tmpLIBS="${LIBS}"
					LIBS="${LIBS} -lvlib"
					AC_TRY_LINK(	[#include <v/mem/longline.h>	],
							[LONGLINE       ll;		],
							[vlib_cv_extraenv_LDFLAGS="none"],
							[
								tmpLDFLAGS="${LDFLAGS}"
								tmpVLIB_xtra=`$VLIB_CONFIG --ldflags`
								LDFLAGS="${LDFLAGS} $tmpVLIB_xtra"
								AC_TRY_LINK(	[#include<v/mem/longline.h>	],
										[LONGLINE	ll;		],
										[vlib_cv_extraenv_LDFLAGS="${tmpVLIB_xtra}"],
										[vlib_cv_extraenv_LDFLAGS=""]
									)

								LDFLAGS="${tmpLDFLAGS}"
							])
					LIBS="${tmpLIBS}"
				])
		if test "$vlib_cv_extraenv_LDFLAGS" = "" ; then
			AC_MSG_ERROR([Unable to link with vlib])
		fi

		if test ! "$vlib_cv_extraenv_LDFLAGS" = "none" ; then
			LDFLAGS="${LDFLAGS} $vlib_cv_extraenv_LDFLAGS"
		fi

		AC_CACHE_CHECK(	[for static version],
				[vlib_cv_path_static],
				[	vlibdir=`${VLIB_CONFIG} --libdir`
					if test -f "${vlibdir}/libvlib.a" ; then
						vlib_cv_path_static="${vlibdir}/libvlib.a"
					else
						vlib_cv_path_static="missing"
					fi])
		if test "${vlib_cv_path_static}" = "missing" ; then
			vlib_static=""
		else
			vlib_static="${vlib_cv_path_static}"
		fi

		if test "${use_static_vlib}" = "1" ; then
			AC_SUBST(LIBVLIB,[${vlib_static}])
		else
			AC_SUBST(LIBVLIB,[-lvlib])
		fi


	]) # AC_LIB_VLIB

