/****************************************/
/*  Class to resolve hostnames          */
/****************************************/
/* Start      : 10/5/98                 */
/* Program ID :                         */
/* Version    : 1.01                    */
/* End        :                         */
/* Last Update: 28/8/2001               */
/* Comments   :                         */
/*                                      */
/****************************************/


#include "vresolv.h"

DNSHostName::DNSHostName()
{
 Reset();
}

DNSHostName::~DNSHostName()
{
}

DNSHostName::DNSHostName(DNSHostName &d)
{
 int n;
 
 AddrType=d.AddrType;
 NameCount=d.NameCount;
 IPCount=d.IPCount;
 
 for (n=0;n<d.NameCount;n++)
  strcpy(Name[n],d.Name[n]);

 for (n=0;n<d.IPCount;n++)
  strcpy(IP[n],d.IP[n]);
}

void DNSHostName::Reset()
{
 int n;
 
 AddrType=-1;
 NameCount=IPCount=0;
 for (n=0;n<_RESOLV_MAX_HOSTNUM;n++)
 {
  Name[n][0]=IP[n][0]=0;
 }
}

void DNSHostName::SetName(char *dst,const  char *src)
{
 strncpy(dst,src,_RESOLV_MAX_HOSTNAME);
 dst[_RESOLV_MAX_HOSTNAME]=0;
}

void DNSHostName::SetData(hostent *h)
{
 int n;
 in_addr ina;

 if (h!=NULL)
 {
  SetName(Name[0],h->h_name);

  for (n=1;((n<_RESOLV_MAX_HOSTNUM) && (h->h_aliases[n-1]!=NULL));n++)
   SetName(Name[n],h->h_aliases[n-1]);
  NameCount=n;

  for (n=0;((n<_RESOLV_MAX_HOSTNUM) && (h->h_addr_list[n]!=NULL));n++)
  {
   ina.s_addr=(*(long *)h->h_addr_list[n]);
   SetName(IP[n],inet_ntoa(ina));
  }
  IPCount=n;
 }
 else
 {
  Reset();
 }
}

#ifdef _DEBUG_

void DNSHostName::ShowData()
{
 int n;
 printf(" {NameCount,IPCount}={%d,%d}\n",NameCount,IPCount);
 for (n=0;n<NameCount;n++)
  printf(" Hostname #%d: %s\n",n,Name[n]);

 for (n=0;n<IPCount;n++)
  printf(" IP #%d: %s\n",n,IP[n]);

 fflush(stdout);
}

#endif

DNSHostName &DNSHostName::operator=(DNSHostName &d)
{
 NameCount=d.NameCount;
 IPCount=d.IPCount;
 AddrType=d.AddrType;
 memcpy(Name,d.Name,sizeof(Name));
 memcpy(IP,d.IP,sizeof(IP));
 return(*this);
}

static int ReverseResolveIsOpen=0;

ReverseResolve::ReverseResolve()
{
 if (ReverseResolveIsOpen!=1)
  ReverseResolveIsOpen=0;
}

ReverseResolve::~ReverseResolve()
{
 if (ReverseResolveIsOpen)
  Close();
}

void ReverseResolve::Open()
{
 if (!ReverseResolveIsOpen)
 {
  sethostent(1);
  ReverseResolveIsOpen=1;
 }
}

void ReverseResolve::Close()
{
 if (ReverseResolveIsOpen)
 {
  endhostent();
  ReverseResolveIsOpen=0;
 }
}

char *ReverseResolve::Resolve(const char *host)
{
 hostent *h;
 u_long addr;

 addr=inet_addr(host);
 
 h=gethostbyaddr((char *)&addr,sizeof(addr),AF_INET);
 SetData(h);
 
#ifdef _DEBUG_
 if (h==NULL)
 {
  ShowData();
  herror("ReverseResolve");
 }
#endif

 return(Name[0]);
}

ReverseResolve &ReverseResolve::operator=(ReverseResolve &d)
{
 DNSHostName::operator=(d);
 return(*this);
}

HostResolve::HostResolve()
{
}

HostResolve::~HostResolve()
{
}

HostResolve::HostResolve(HostResolve &d)
{
 addr=d.addr;
}

char *HostResolve::Resolve(const char *name)
{
 hostent *h;
 char *s;

 h=gethostbyname(name);
 SetData(h);

#ifdef _DEBUG_
 if (h==NULL)
 {
  ShowData();
  herror("HostResolve");
 }
#endif

// addr.s_addr=(*h->h_addr);

 s=NULL;
 if (IP[0]!=NULL)
  s=IP[0];
 return(IP[0]);
}

HostResolve HostResolve::operator=(HostResolve &d)
{
 DNSHostName::operator=(d);
 memcpy(&addr,&d.addr,sizeof(addr));

 return(*this);
}

HostName::HostName()
{
 hname=NULL;
}

HostName::~HostName()
{
}

HostName::HostName(HostName &d)
{
/* rr=d.rr;
 hr=d.hr;
 if (d.hname==&d.rr)
  hname=&rr;
 else
  hname=&hr;*/
 operator=(d);
}

char *HostName::Resolve(const char *host)
{
 int n;
 char *s;
 
 n=inet_addr(host);
 
 if (n==-1)
 {
  s=hr.Resolve(host);
  hname=&hr;
 } else
 {
  s=rr.Resolve(host);
  hname=&rr;
 }

 return(s);
}

char *HostName::GetIP(char *host)
{
 int n;
 char *s;

 n=inet_addr(host);
 
 if (n==-1)
 {
  s=hr.Resolve(host);
  hname=&hr;
 } else
 {
  rr.Resolve(host);
  hname=&hr;
  s=host;
 }

 return(s);
}

HostName HostName::operator=(HostName &d)
{
 rr=d.rr;
 hr=d.hr;
 if (d.hname==&d.rr)
  hname=&rr;
 else
  hname=&hr;
 return(*this);	// Added 28/8/2001
}


