/****************************************/
/*					*/
/****************************************/
/* Start      : 08/09/2002		*/
/* Version    : 1.00			*/
/* Last Update: 08/09/2002		*/
/****************************************/

#include "selector.h"

#include "v/mem/fixedelementlist.cc"
#include "v/misc/nkeytable.cc"
#include "v/minmax.h"

#include "sys/time.h"
#include "sys/types.h"
#include "unistd.h"

//////////////////////////////////////////////////////////////////////
// SELECTOR
//////////////////////////////////////////////////////////////////////

SELECTOR::SELECTOR()
{
}

SELECTOR::~SELECTOR()
{
}

SELECTOR::SELECTOR(SELECTOR &d)
{
	ddd(d);
}

SELECTOR SELECTOR::operator=(SELECTOR &d)
{
	ddd(d);

	return (*this);
}

void	SELECTOR::ddd(SELECTOR &d)
{
}

int	SELECTOR::add_fd(int fd)
{
	CLNT	*pclnt;

	pclnt=clients[fd];
	if (pclnt!=NULL)
		return(-1);

	CLNT	clnt;

	clnt.fd=fd;
	clients.Add(fd,&clnt);

	return(0);
}

int	SELECTOR::del_fd(int fd)
{
	CLNT	*pclnt;

	pclnt=clients[fd];
	if (pclnt==NULL)
		return(-1);

	clients.Del(fd);

	return(0);
}

int	SELECTOR::set_class(int fd, SELECTOR_BASE *cls)
{
	CLNT	*pclnt;

	pclnt=clients[fd];
	if (pclnt==NULL)
		return(-1);

	pclnt->cls=cls;
	pclnt->cls_one=NULL;
	pclnt->onRead=NULL;
	pclnt->onWrite=NULL;
	pclnt->onExcept=NULL;
	pclnt->onConnect=NULL;

	return(0);
}

int	SELECTOR::set_class(int fd, SELECTOR_BASE_ONE *cls_one)
{
	CLNT	*pclnt;

	pclnt=clients[fd];
	if (pclnt==NULL)
		return(-1);

	pclnt->cls=NULL;
	pclnt->cls_one=cls_one;
	pclnt->cls_one->fd=fd;
	pclnt->onRead=NULL;
	pclnt->onWrite=NULL;
	pclnt->onExcept=NULL;
	pclnt->onConnect=NULL;

	return(0);
}
int	SELECTOR::set_hook_read(int fd, SELECTOR_HOOK *hook)
{
	CLNT	*pclnt;

	pclnt=clients[fd];
	if (pclnt==NULL)
		return(-1);

	pclnt->cls=NULL;
	pclnt->cls_one=NULL;
	pclnt->onRead=hook;

	return(0);
}

int	SELECTOR::set_hook_write(int fd, SELECTOR_HOOK *hook)
{
	CLNT	*pclnt;

	pclnt=clients[fd];
	if (pclnt==NULL)
		return(-1);

	pclnt->cls=NULL;
	pclnt->cls_one=NULL;
	pclnt->onWrite=hook;

	return(0);
}

int	SELECTOR::set_hook_except(int fd, SELECTOR_HOOK *hook)
{
	CLNT	*pclnt;

	pclnt=clients[fd];
	if (pclnt==NULL)
		return(-1);

	pclnt->cls=NULL;
	pclnt->cls_one=NULL;
	pclnt->onExcept=hook;

	return(0);
}

int	SELECTOR::set_hook_connect(int fd, SELECTOR_HOOK *hook)
{
	CLNT	*pclnt;

	pclnt=clients[fd];
	if (pclnt==NULL)
		return(-1);

	pclnt->cls=NULL;
	pclnt->cls_one=NULL;
	pclnt->onConnect=hook;

	return(0);
}

int	SELECTOR::clear_hooks(int fd)
{
	CLNT	*pclnt;

	pclnt=clients[fd];
	if (pclnt==NULL)
		return(-1);

	pclnt->onRead=NULL;
	pclnt->onWrite=NULL;
	pclnt->onExcept=NULL;
	pclnt->onConnect=NULL;

	return(0);
}

int	SELECTOR::doit(int timeout)
{
	return(doit_select(timeout));
}

int	SELECTOR::doit_select(int timeout)
{
	u_32			key;
	CLNT			*pclnt;
	fd_set			fdr,fdw,fde;
	struct timeval		tv,*ptv;
	int			ret,sret;
	int			fdmax;
	FixedElementList<int>	to_remove;
	int			*pn;

	to_remove.SetUnique(1);

// Set fd_sets

	clients.Rewind();

	FD_ZERO(&fdr);
	FD_ZERO(&fdw);
	FD_ZERO(&fde);

	fdmax=-1;

	while ((key=clients.Get_Next())!=NK_INVALID)
	{
		pclnt=clients[key];
		FD_SET(key, &fde);
/*		if (pclnt->cls!=NULL)	// If there is a class
		{
			if (pclnt->wait & SB_WAIT_READ)
				FD_SET(key, &fdr);
			if ((pclnt->wait & SB_WAIT_WRITE) || (pclnt->wait & SB_WAIT_CONNECT))
				FD_SET(key, &fdw);

		} else if (pclnt->cls_one!=NULL)
		{
			if (pclnt->cls_one->WaitRead)
				FD_SET(key, &fdr);
			if ((pclnt->cls_one->WaitWrite) || (pclnt->cls_one->WaitConnect))
				FD_SET(key, &fdw);
		} else
		{
			if (pclnt->onRead!=NULL)
				FD_SET(key, &fdr);
			if ((pclnt->onWrite!=NULL) || (pclnt->onConnect!=NULL))
				FD_SET(key, &fdw);
		}*/
		if (pclnt->NeedRead())
			FD_SET(key, &fdr);
		if (pclnt->NeedWrite())
			FD_SET(key, &fdw);

		fdmax=MAX((s_32)key,fdmax);
	}

	fdmax++;

// Do the select
	if (timeout==-1)
	{
		ptv=NULL;
	} else
	{
		tv.tv_sec=timeout/1000;		// seconds
		tv.tv_usec=(timeout%1000)*1000;	// microseconds
		ptv=&tv;
	}

	sret=select(fdmax, &fdr, &fdw, &fde, ptv);
	if (sret>0)
	{
		clients.Rewind();
		while ((key=clients.Get_Next())!=NK_INVALID)
		{
			pclnt=clients[key];
			ret=0;
			if (FD_ISSET(key,&fde))
				ret=pclnt->CallExcept();
			if (ret!=-1)
			{
				if (FD_ISSET(key,&fdr))
					ret=pclnt->CallRead();
			}
			if (ret!=-1)
			{
				if (FD_ISSET(key,&fdw))
					ret=pclnt->CallWrite();
			}
			if (ret==-1)
			{
				to_remove.Add(key);
			}
		}
	}

// Remove entries marked to be removed
	to_remove.Rewind();
	while ((pn=to_remove.Get_Next())!=NULL)
	{
		clients.Del(*pn);
	}

	to_remove.Clear();

	return(sret);
}

int	SELECTOR::set_class_read(int fd, int n)
{
	CLNT	*pclnt;

	pclnt=clients[fd];
	if (pclnt==NULL)
		return(-1);

	if (n)
		pclnt->wait|=SB_WAIT_READ;
	else
		pclnt->wait&=~SB_WAIT_READ;

	return(0);
}

int	SELECTOR::set_class_write(int fd, int n)
{
	CLNT	*pclnt;

	pclnt=clients[fd];
	if (pclnt==NULL)
		return(-1);

	if (n)
		pclnt->wait|=SB_WAIT_WRITE;
	else
		pclnt->wait&=~SB_WAIT_WRITE;

	return(0);
}

int	SELECTOR::set_class_connect(int fd, int n)
{
	CLNT	*pclnt;

	pclnt=clients[fd];
	if (pclnt==NULL)
		return(-1);

	if (n)
		pclnt->wait|=SB_WAIT_CONNECT;
	else
		pclnt->wait&=~SB_WAIT_CONNECT;

	return(0);
}

int	SELECTOR::clear_class_rwc(int fd)
{
	CLNT	*pclnt;

	pclnt=clients[fd];
	if (pclnt==NULL)
		return(-1);

	pclnt->wait=0;

	return(0);
}

int	SELECTOR::remove_class(int fd)
{
	CLNT	*pclnt;

	pclnt=clients[fd];
	if (pclnt==NULL)
		return(-1);

	clients.Del(fd);

	return(0);
}

//////////////////////////////////////////////////////////////////////
// SELECTOR::CLNT
//////////////////////////////////////////////////////////////////////

SELECTOR::CLNT::CLNT()
{
	fd=-1;
	wait=0;

	cls=NULL;
	cls_one=NULL;
	onRead=NULL;
	onWrite=NULL;
	onExcept=NULL;
	onConnect=NULL;
}

SELECTOR::CLNT::~CLNT()
{
}

SELECTOR::CLNT::CLNT(SELECTOR::CLNT &d)
{
	ddd(d);
}

SELECTOR::CLNT SELECTOR::CLNT::operator=(SELECTOR::CLNT &d)
{
	ddd(d);

	return (*this);
}

void	SELECTOR::CLNT::ddd(SELECTOR::CLNT &d)
{
	fd=d.fd;
	wait=d.wait;

	cls=d.cls;
	cls_one=d.cls_one;
	onRead=d.onRead;
	onWrite=d.onWrite;
	onExcept=d.onExcept;
	onConnect=d.onConnect;
}

int	SELECTOR::CLNT::CallRead()
{
	int	ret;

	if (cls!=NULL)
		ret=cls->doRead(fd);
	else if (cls_one!=NULL)
		ret=cls_one->doRead();
	else
		ret=(onRead==NULL ? -1 : onRead(fd));

	return(ret);
}

int	SELECTOR::CLNT::CallWrite()
{
	int	ret;

	if (cls!=NULL)
	{
		if (wait & SB_WAIT_CONNECT)
			ret=cls->doConnect(fd);
		else
			ret=cls->doWrite(fd);
	} else if (cls_one!=NULL)
	{
		if (cls_one->WaitConnect)
			ret=cls_one->doConnect();
		else
			ret=cls_one->doWrite();
	} else
	{
		if (onConnect==NULL)
		{
			ret=(onWrite==NULL ? -1 : onWrite(fd));
		} else
		{
			ret=onConnect(fd);
		}
	}

	return(ret);
}

int	SELECTOR::CLNT::CallExcept()
{
	int	ret;

	if (cls==NULL)
		ret=cls->doExcept(fd);
	else if (cls_one!=NULL)
		ret=cls_one->doExcept();
	else
		ret=(onExcept==NULL ? -1 : onExcept(fd));

	return(ret);
}

int	SELECTOR::CLNT::NeedRead()
{
	int	ret=0;

	if (cls!=NULL)	// If there is a class
	{
		if (wait & SB_WAIT_READ)
			ret=1;
	} else if (cls_one!=NULL)
	{
		if (cls_one->WaitRead)
			ret=1;
	} else
	{
		if (onRead!=NULL)
			ret=1;
	}

	return(ret);
}

int	SELECTOR::CLNT::NeedWrite()
{
	int	ret=0;

	if (cls!=NULL)	// If there is a class
	{
		if ((wait & SB_WAIT_WRITE) || (wait & SB_WAIT_CONNECT))
			ret=1;
	} else if (cls_one!=NULL)
	{
		if ((cls_one->WaitWrite) || (cls_one->WaitConnect))
			ret=1;
	} else
	{
		if ((onWrite!=NULL) || (onConnect!=NULL))
			ret=1;
	}

	return(ret);
}

//////////////////////////////////////////////////////////////////////
// SELECTOR_BASE
//////////////////////////////////////////////////////////////////////
