/****************************************/
/*					*/
/****************************************/
/* Start      : 24/08/2001		*/
/* Program ID : 			*/
/* Version    : 1.00			*/
/* End        : 			*/
/* Last Update: 24/08/2001		*/
/* Comments   : 			*/
/*                                      */
/****************************************/

#include "v/lconfig.h"
#include "cgi.h"
#include "v/misc/keytable2.cc"
#include "v/mem/dlist3.cc"

#ifdef HAVE_STRING_H
#	include<string.h>
#else
#	include<strings.h>
#endif

#ifndef HAVE_STRCASECMP
#	define strcasecmp stricmp
#endif

#define METHOD_GET	"GET"
#define METHOD_POST	"POST"
#define METHOD_HEAD	"HEAD"

#define SERVER_SOFTWARE		"SERVER_SOFTWARE"
#define SERVER_NAME		"SERVER_NAME"
#define SERVER_PROTOCOL		"SERVER_PROTOCOL"
#define SERVER_PORT		"SERVER_PORT"

#define PATH_INFO		"PATH_INFO"
#define PATH_TRANSLATED		"PATH_TRANSLATED"

#define REMOTE_HOST		"REMOTE_HOST"
#define REMOTE_ADDR		"REMOTE_ADDR"
#define REMOTE_USER		"REMOTE_USER"
#define REMOTE_IDENT		"REMOTE_IDENT"
#define AUTH_TYPE		"AUTH_TYPE"

#define SCRIPT_NAME		"SCRIPT_NAME"
#define GATEWAY_INTERFACE	"GATEWAY_INTERFACE"
#define QUERY_STRING		"QUERY_STRING"
#define REQUEST_METHOD		"REQUEST_METHOD"
#define CONTENT_TYPE		"CONTENT_TYPE"
#define CONTENT_LENGTH		"CONTENT_LENGTH"

#define HTTP_ACCEPT		"HTTP_ACCEPT"
#define HTTP_USER_AGENT		"HTTP_USER_AGENT"

#define CONTENT_TYPE_ENCODED	"application/x-www-form-urlencoded"

// Proprietary headers
#define HTTP_COOKIE		"HTTP_COOKIE"

// Send-back headers
#define HEADER_CONTENT_TYPE	"Content-type"
#define HEADER_SET_COOKIE	"Set-Cookie"

struct	{ char *name; } env_vars[] =
		{ {SERVER_SOFTWARE}, {SERVER_NAME}, {SERVER_PROTOCOL}, {SERVER_PORT},
		  {PATH_INFO}, {PATH_TRANSLATED},
		  {REMOTE_HOST}, {REMOTE_ADDR}, {REMOTE_USER}, {REMOTE_IDENT}, {AUTH_TYPE},
		  {SCRIPT_NAME}, {GATEWAY_INTERFACE}, {QUERY_STRING}, {REQUEST_METHOD}, {CONTENT_TYPE}, {CONTENT_LENGTH},
		  {HTTP_ACCEPT}, {HTTP_USER_AGENT},
		  {HTTP_COOKIE},		// proprietary
		  {NULL} };


CGI::CGI()
{
	error=0;
	no_contenttype=0;
	method=method_unknown;
	set_content(CGI_CONTENT_TEXTHTML);
}

CGI::~CGI()
{
}

void	CGI::init()
{
	load_env();
	get_cookies();
	parse();
}

/*
	Parse a string converting %$$ to characters
	and store it in dst
*/
void	CGI::decode_str(const char *str, LONGLINE &dst)
{
	const char	*p;
	char		num[3];
	int		n;

	num[2]=0;
	p=str;
	dst="";
	while (*p)
	{
		switch(*p)
		{
			case '%':
				p++;
				if ((*p!=0) && (*(p+1)!=0) &&
					(strchr("1234567890abcdefABCDEF",*p)!=NULL) &&
					(strchr("1234567890abcdefABCDEF",*(p+1))!=NULL))
  				{               // ignore %00
					num[0]=*p;
					num[1]=*(p+1);
//					n=atoi(num);	// we have the number
					n=strtol(num,NULL,16);
					p+=2;
					if (n) dst+=(char)n;	// ignore %00
				} else dst+='%';
				break;
			case '+':
				dst+=' ';
				p++;
				break;
			default:
				dst+=*p;
				p++;
				break;
		}
	}
}

void	CGI::encode_str(const char *str, LONGLINE &dst)
{
	const char	*p;
	char		num[5];

	p=str;
	dst="";
	num[2]=0;
	while (*p)
	{
		if (isalnum(*p))
			dst+=*p;
   		else
		{
			sprintf(num,"%02X",*p);
			dst+=num;
		}
		p++;
	}
}

/*
	Parse a line. if needdecode==1 then decode it also
*/
void	CGI::parse_qsline(const char *line, int needdecode)
{
	PARSE		pars1,pars2;
	char		*p,*p2;
	char		*name, *value;
	LONGLINE	ll;
	int		n=0;

	pars1.flags=PARSEFLAG_IGNOREEMPTY;
	pars1.AddSep('&');
	pars1.Set(line);

	pars2.AddSSep('=');

	while ((p=pars1[n])!=NULL)
	{
		pars2.Set(p);
		p2=pars2[0]; if (p2==NULL) name=NULL;
		else
		{
			name=p2;
			p2=pars2[1]; if (p2==NULL) ll=""; else
			{
				value=p2;
				if (needdecode)
					decode_str(value,ll);	// decode it
				else
					ll=value;
			}
			vars.Add(name,ll);
		}
		n++;
	}
}

void	CGI::parse_qs()
{
	parse_qsline(env[QUERY_STRING]->p);
}

void	CGI::parse_stdin()
{
	char		buf[513];
	int		n,needdecode=0;
	LONGLINE	ll,*pll;

	buf[512]=0;
	ll="";
	pll=env[CONTENT_TYPE];
	if ((pll!=NULL) && (*pll==CONTENT_TYPE_ENCODED))
		needdecode=1;

	while (!feof(stdin))
	{
		if (fgets(buf,513,stdin)!=NULL)
		{
			all_input+=buf;
			n=strlen(buf);
			ll+=buf;
			if (buf[n-1]=='\n')
			{
				ll--;
				parse_qsline(ll.p,needdecode);
				ll="";
			}
		}
	}
	if (ll!="")
	{
		parse_qsline(ll.p,needdecode);
	}
}

void	CGI::get_cookies()
{
//	LONGLINE	ll,*pll,
	LONGLINE	ll, *line;
	char		*p=NULL, *p1, *p2,*p3;
	int		done=0;
	PARSE		pars1;

	LONGLINE	name,value;
	int		quoted;

	line=env[HTTP_COOKIE];

//	pars1.flags=PARSEFLAG_IGNOREEMPTY;
//	pars2.flags=PARSEFLAG_IGNOREEMPTY;

	if (line==NULL) return;


	pars1.AddSSep('=');
	ll=line->p;

	while (!done)
	{
		pars1.Set(ll);
		p1=pars1[0];
		p2=pars1[1];
		if ((p1) && (*p1))
		{
			PARSE	pars2;

			p3=&p1[strlen(p1)-1];
			while (*p3==' ') p3--;	// ignore spaces
//			name=p1;
			name.Strncpy(p1,(p3-p1)+1);
			value="";
			if ((p2) && (*p2))	// there is a value
			{
				while (*p2==' ') p2++;	//ignore spaces
				pars2.AddSSep(';');
				pars2.AddSSep(',');
				if (*p2=='\"')	// quoted ?
				{
//					pars2.AddSSep('\"');
					quoted=1;
					p2++;	// dont't bother with the starting "
     				} else
				{
					quoted=0;
				}
				pars2.Set(p2);
				p=pars2[0];
				p3=&p[strlen(p)-1];
				while (*p3==' ') p3--;
				if (quoted)
				{
				 if (*p3!='\"') return; //error
				 else p3--;
				}
				value.Strncpy(p,(p3-p)+1);

				p=pars2[1];
				if (p)
				{
					while (*p==' ') p++;	// more spaces
					if (*p==0) done=1;	// the end ?
				} else done=1;
			}
			if (name[0]!='$')	// TODO: Handle this
			{
				decode_str(value.p,ll);
				cookies.Add(name,ll);
			}
			if (p) ll=p;
		} else done=1;
	}

/*	while (!done)
	{
		name="";
		while (*p==' ') p++;	//ignore starting spaces
		p2=strchr(p,'=');
		p3=strchr(p,' ');
		if (*p2)
		{
			if (*p3) p2=MIN(p2,p3);
		} else p2=p3;
		if (*p2)	// found
		{
			name.Strncat(p,p2-p);
			p=p2;
			while (*p==' ') p++;	// ignore spaces before =
			if (*p!='=') done=1; // error
			else
			{
				p++;
				while (*p==' ') p++;	// ignore spaces after =
				p2=strchr(p,';');
				if (*p2)
				{
				} else value=p;	// no more stuff
			}
		}
		if (*p==0) done=1;
	}

	pars1.flags=PARSEFLAG_IGNOREEMPTY;
	pars1.AddSep(';');
	pars1.AddSep(',');
	pars1.Set(line->p);

	pars2.AddSSep('=');

	while ((p=pars1[n])!=NULL)
	{
//		while ((*p!=0) && (*p==' ')) p++;	// Ignore starting spaces
		while (*p==' ') p++;

		pars2.Set(p);

		p2=pars2[0]; if (p2!=NULL)
		{
			name=p2;
			p2=pars2[1]; if (p2!=NULL)
			{
				if (*name!='$')
				{
					value=p2;
					if (*p2=='\"') value++;	// check for quoted string
					decode_str(value,ll);
					if (*p2=='\"') ll--;
					cookies.Add(name,ll);
				}	//TODO.. handle $
			}
		}

		n++;
	}*/
}

void	CGI::set_header(const char *name, const char *value)
{
	LONGLINE	ll;

	ll=name;
	ll+=": ";
	ll+=value;

	headers.PushL(ll);
}

void	CGI::set_cookie(const char *name, const char *value, const char *domain,
			time_t maxage, const char *path, int secure, int version,
			const char *comment)
{
	LONGLINE	ll,ll2;

	ll=name;
	ll+="=";
	encode_str(value,ll2);
	ll+=ll2;
	ll+="; Version="; ll+=(long)version;
	if (path!=NULL)	{ ll+="; Path="; ll+=path; }
	if (domain!=NULL) { ll+="; Domain="; ll+=domain; }
	ll+="; Max-Age="; ll+=(long)maxage;
	if (secure) ll+="; Secure";
	if (comment!=NULL) { ll+="; Comment=\""; ll+=comment; ll+="\""; }

	set_header(HEADER_SET_COOKIE, ll.p);
}

void	CGI::set_redirect(const char *url)
{
	no_contenttype=1;
	set_header("Location",url);
}

void	CGI::set_content(const char *content)
{
	this->content=content;
}

void	CGI::print_headers()
{
	LONGLINE	*pll;

	if (!no_contenttype)
	{
		fprintf(stdout,"%s: %s\n",HEADER_CONTENT_TYPE, content.p);
	}
	headers.Rewind();
	while ((pll=headers.Get_Next())!=NULL)
	{
		fprintf(stdout,"%s\n",pll->p);
	}
	fprintf(stdout,"\n");
	fflush(stdout);
}

void	CGI::parse()
{
	char		*p;
	LONGLINE	*pll;

	all_input="";
	pll=env[REQUEST_METHOD];
	if (*pll=="") error=1;
	else
	{
		p=pll->p;
		if (!strcasecmp(p,METHOD_POST)) method=method_post;
		else if (!strcasecmp(p,METHOD_GET)) method=method_get;
		else if (!strcasecmp(p,METHOD_HEAD)) method=method_head;
		else error=1;
	}

	if (!error)
	{
		switch(method)
		{
			case method_head:	// head requests should not return a body
			case method_get:
				parse_qs();
				break;
			case method_post:
				parse_qs();
				parse_stdin();
				break;
		}
	}
}

void	CGI::load_env()
{
	int	n=0;
	char	*p,*p2;

	while ((p2=env_vars[n].name)!=NULL)
	{
		p=getenv(p2);
		if (p!=NULL) env.Add(p2,p); else env.Add(p2,"");
		n++;
	}
}

void	CGI::clear()
{
	env.ClearTable();
	vars.ClearTable();
	cookies.ClearTable();
	content=CGI_CONTENT_TEXTHTML;
	headers.ClearList();
	method=method_unknown;
}
