/****************************************/
/*					*/
/****************************************/
/* Start      : 24/08/2001		*/
/* Program ID : 			*/
/* Version    : 1.00			*/
/* End        : 			*/
/* Last Update: 24/08/2001		*/
/* Comments   : 			*/
/*                                      */
/****************************************/

#ifndef _VLIB_V_HTML_CGI_H
#define _VLIB_V_HTML_CGI_H

#include "stdlib.h"

#include "sys/types.h"
#include "time.h"

#include "v/mem/longline.h"
#include "v/misc/keytable2.h"
#include "v/string/parse.h"

#ifndef CGI_COOKIE_EXPIRE
#define	CGI_COOKIE_EXPIRE	24*60*60	// expire in 1 day by default
#endif

#define	CGI_CONTENT_TEXTHTML	"text/html"

/*
	This will NOT handle %00 . Those will NOT be appended to the strings,
	parsing will continue just like they didn't exist.
*/

/*
	TODO: Handle $Version $Path $Domain
*/

class	CGI
{
 private:
 	DL_List<LONGLINE>	headers;	// the headers that are going to be sent
	LONGLINE		content;
	int			no_contenttype;	// if no Content-type header should be sent (In case of redirect)
 public:
 	enum METHOD
	{
		method_get, method_post, method_head, method_unknown
	} method;

	int			error;
	KEYTABLE2<LONGLINE>	env;
	KEYTABLE2<LONGLINE>	vars;
	KEYTABLE2<LONGLINE>	cookies;
	LONGLINE		all_input;	// This holds the whole input from stdin (if any)
 private:
 	void	check_env	();
	void	load_env	();
	void	parse		();
	void	decode_str	(const char *str, LONGLINE& dst);	// decode a string
	void	encode_str	(const char *str, LONGLINE& dst);	// encode a string
	void	parse_stdin	();
	void	parse_qsline	(const char *line, int needdecode=1);
	void	parse_qs	();	// parse QUERY_STRING
 	void	get_cookies	();
 public:
 		CGI		();
		~CGI		();
	void	init		();
	void	set_cookie	(const char *name, const char *value, const char *domain=NULL,
				time_t maxage=CGI_COOKIE_EXPIRE, const char *path=NULL,
				int secure=0, int version=1, const char *comment=NULL);
	void	set_header	(const char *name, const char *value);
	void	set_content	(const char *content);
	void	set_redirect	(const char *url);	// Redirect the client to this URL. Suppress Content-type header
	void	print_headers	();
	void	clear		();	// free the memory - deleting the CGI will also work
};

/*
	Usage:
		int main()
		{
			CGI	cgi;

			cgi.init();
			cgi.set_cookie("cookie","value",".domain.gr",10, // expire time in seconds
					"/~home");		// set_cookie is optional
			cgi.set_header("header","value");	// (optional) to set any aditional headers
			cgi.print_headers();		// display the headers

			... The main program ...
		}
*/


#endif

