/****************************************/
/*  Byte Queue (header)                 */
/****************************************/
/* Start      : 23/12/2000              */
/* Program ID :                         */
/* Version    : 1.00                    */
/* End        :                         */
/* Last Update: 23/21/2000              */
/* Comments   : FIFO                    */
/*                                      */
/****************************************/

#ifndef _VLIB_V_MEM_BYTEQUEUE_H
#define _VLIB_V_MEM_BYTEQUEUE_H

#include "v/mem/dlist3.h"
#include "v/mem/longline.h"

#include "stdlib.h"

class ByteQueue
{
 protected:
	char *			begin;
	DL_List<char *>		middle;
	char *			end;	// This is used only if begin is full

	size_t	beg_ofs;	// where data begin in begin
	size_t	end_data;	// bytes in end
	size_t	buflen;		// total size of data

 public:
 private:
	void		ddd			(ByteQueue &d);
	const char 	*append_buf_begin	(const char *buf, size_t len, size_t *appended);
	const char	*append_buf_middle	(const char *buf, size_t len, size_t *appended);
	const char 	*append_buf_end		(const char *buf, size_t len, size_t *appended);
	void		append_buf		(const char *buf, size_t len);
	void		create_new_end		(const char *buf, size_t len);
	size_t		read_start		(char *buf, size_t maxlen);
	size_t		discard_start		(size_t maxlen);
	void		clearall		();
	long		get_line_sz		();
 public:
			ByteQueue	();
	virtual 	~ByteQueue	();
			ByteQueue	(ByteQueue &d);
	ByteQueue	operator=	(ByteQueue &d);

	void		Push		(const char *buf, size_t len);	// Add data to the FIFO
	size_t		Pop		(char *buf, size_t maxlen);	// Get fata from the FIFO (data are deleted)
	void		UnPop		(const char *buf, size_t len);	// Put back the poped data.. They will be returned with the next Pop()
	size_t		PopNoErase	(char *buf, size_t maxlen);	// Pop some data, without removing the fromt he FIFO
	size_t		Discard		(size_t len);			// Discard len bytes
	size_t		Size		();
	void		Clear		();				// Clear list
	long		PopLineNoErase	(LONGLINE &line);		// Get a line from the fifo. Remove trailing \n\r.
	long		DiscardLine	();				// Discard the first line from the fifo.
	long		PopLine		(LONGLINE &line);		// Get a line from the fifo and discard it.
};


#endif

