/****************************************/
/*					*/
/****************************************/
/* Start      : 31/08/2002		*/
/* Version    : 1.00			*/
/* Last Update: 31/08/2002		*/
/****************************************/

#include "fixedelementlist.h"

#include "string.h"
#include "stdlib.h"

#define	BLOCKSIZE	20

template<class T> FixedElementList<T>::FixedElementList()
{
	buf=NULL;
	bs=BLOCKSIZE;
	size=0;
	count=0;
	pos=0;
	unique=0;
	SetSize(bs * 1);
}

template<class T> void FixedElementList<T>::vvv(FixedElementList &p)
{
	bs=p.bs;

	SetSize(p.size);
	memcpy(buf, p.buf, p.count * sizeof(T));

	count=p.count;
	pos=p.pos;
	unique=p.unique;
}

template<class T> FixedElementList<T>::FixedElementList(FixedElementList &p)
{
	buf=NULL;
	bs=BLOCKSIZE;
	size=0;
	count=0;
	pos=0;
	unique=1;
	SetSize(bs * 1);

	vvv(p);
}

template<class T> FixedElementList<T>::~FixedElementList()
{
	free(buf);
}

template<class T> FixedElementList<T> FixedElementList<T>::operator=(FixedElementList<T> &p)
{
	vvv(p);

	return(*this);
}

template<class T> void FixedElementList<T>::SetSize(size_t elements)
{
	size_t	ns;

	ns=elements/bs;
	if (((elements % bs) > 0) || (ns==0)) ns++;

	ns*=bs;

	if ((ns==size) && (buf!=NULL))	// No change is required
		return;

	if (buf==NULL)
	{
		buf=(T *)malloc(sizeof(T) * ns);
	} else
	{
		buf=(T *)realloc(buf,sizeof(T) * ns);
	}

	size=ns;
}

template<class T> T FixedElementList<T>::operator[](size_t l)
{
	return(Get(l));
}

template<class T> T FixedElementList<T>::Get(size_t l)
{
	T	ret;

	if (l<count)
		ret=buf[l];

	return(ret);
}

template<class T> void FixedElementList<T>::Add(T l)
{
	if (!unique || !Exist(l))
	{
		count++;
		SetSize(count);
		buf[count-1]=l;
	}
}

template<class T> void FixedElementList<T>::Del(T l)
{
	long	idx;

	idx=Index(l);
	if (idx!=-1)	// if it exists
	{
		DelIndex(idx);
	}
}

template<class T> void FixedElementList<T>::DelIndex(size_t idx)
{
	if (idx>=count)
		return;

	if (idx!=count-1)	// if it is not the last element
	{
		memmove(&(buf[idx]), &(buf[idx+1]), (count-idx-1) * sizeof(T));
	}
	count--;
	SetSize(count);
}


template<class T> int FixedElementList<T>::Exist(T l)
{
	long	idx;

	idx=Index(l);

	return((idx==-1 ? 0 : 1));
}

template<class T> long FixedElementList<T>::Index(T l)
{
	size_t	idx=0;

	while (idx<count)
	{
		if (buf[idx]==l)
			return(idx);

		idx++;
	}


	return(-1);
}

template<class T> void FixedElementList<T>::Clear()
{
	SetSize(1);
	count=0;
	pos=0;
}

template<class T> void FixedElementList<T>::Rewind()
{
	pos=0;
}

template<class T> T *FixedElementList<T>::Get_Next()
{
	T	*ret;

	if (pos>=count)
		return(NULL);

	ret=&buf[pos];
	pos++;

	return(ret);
}

template<class T> void FixedElementList<T>::Set(size_t idx, T l)
{
	if (idx>=count)
		return;
		
	buf[idx]=l;
}

template<class T> void FixedElementList<T>::SetUnique(int n)
{
	int	u;
	int	m,z;
	T	p;

	u=(n ? 1 : 0);
	if ((u!=unique) && (u==1))
	{
		// Remove duplicates

		m=0;
		for (m=count-1;m>=0;m--)
		{
			p=buf[m];
			do {
				z=Index(p);
				if ((z!=m) && (z>=0))
				{
					DelIndex(z);
					m--;
				}
			} while ((z!=m) && (z>=0));
		}
	}
	unique=u;
}

