/****************************************/
/*  Long line class                     */
/****************************************/
/* Start      : 12/8/98                 */
/* Program ID :                         */
/* Version    : 1.05                    */
/* End        :                         */
/* Last Update: 04/06/2002              */
/* Comments   : Class for very long     */
/*              lines with dynamic      */
/*              memory allocation       */
/****************************************/

/*
	4/6/2002	v1.05:		Perfomance improvement in strcat
					Changed resize algorithm ot use a multiplication factor when longline becomes big
*/

#ifndef _VLIB_V_MEM_LONGLINE_CC
#define _VLIB_V_MEM_LONGLINE_CC
#include "longline.h"
#include "v/lconfig.h"

#ifndef _LONGLINE_BLOCKSIZE
#	error _LONGLINE_BLOCKSIZE is not defined. configure failed ?
#endif

#define _LONGMEM_BLOCKSIZE      _LONGLINE_BLOCKSIZE

LONGLINE::LONGLINE()
{
 Init();
}

#include "stdio.h"

void LONGLINE::Init()
{
 len=0;
 p=NULL;
 size=0;
 Resize(1);
 Strcpy("");
}

LONGLINE::~LONGLINE()
{
 if (p!=NULL)
  free(p);
}


LONGLINE::LONGLINE(const char *str)
{
 Init();
 Strcpy(str);
}

LONGLINE::LONGLINE(const LONGLINE &p1)
{
 Init();
 Strcpy(p1);
}


void LONGLINE::Resize(long n)
{
 char *p2;

 if (n!=size)
 {
  if (p!=NULL)
  {
   p2=(char *)realloc(p,(size_t)n*_LONGLINE_BLOCKSIZE);
  } else
   p2=(char *)malloc((size_t)n*_LONGLINE_BLOCKSIZE);
  p=p2;
  size=n;
  len=MIN(len,((size_t)n*_LONGLINE_BLOCKSIZE)-1);
  p[len]=0;
 }
}

char *LONGLINE::Strcat(const char *src)
{
 size_t len2;
 size_t sum;

 if (src!=NULL)
 {
  len2=strlen(src);
  sum=len2+len;
  Resize(CalcNewSize(sum));
  if (len>5)
	strcat(p+len-2,src);	// 4/6/2002 - speed-up
  else
  	strcat(p,src);
  len=sum;
 }
 return(p);
}

char *LONGLINE::Strcat(const LONGLINE& p2)
{
 Strcat(p2.p);

 return(p);
}

char *LONGLINE::Strncat(const char *src, size_t s)
{
 size_t sum;

 if (src!=NULL)
 {
  sum=s+len;
  Resize(CalcNewSize(sum));
  strncat(p,src,s);
  len=sum;
 }
 return(p);
}

char *LONGLINE::Strncat(const LONGLINE& p2, size_t s)
{
 Strncat(p2.p,s);

 return(p);
}

char *LONGLINE::Strncpy(const char *src, size_t s)
{
 if (src!=NULL)
 {
  Resize(CalcNewSize(s));
  strncpy(p,src,s);
  p[s]=0;
  len=s;
 }
 return(p);
}

char *LONGLINE::Strncpy(const LONGLINE& p2, size_t s)
{
 Strncpy(p2.p,s);

 return(p);
}

char *LONGLINE::Strcpy(const char *src)
{
 size_t len2;

 if (src!=NULL)
 {
  len2=strlen(src);
  Resize(CalcNewSize(len2));
  strcpy(p,src);
  len=len2;
 } else Strcpy("");

 return(p);
}

char *LONGLINE::Strcpy(const LONGLINE& p2)
{
 Strcpy(p2.p);

 return(p);
}

int LONGLINE::Strcmp(const LONGLINE& p2) const
{
 return(strcmp(p,p2.p));
}

int LONGLINE::Strcmp(const char *p2) const
{
	int ret;

	if (p2==NULL) ret=-1;		// fixed 9/7/2001
	else ret=strcmp(p,p2);

	return(ret);
}

/*
	When size is more than _BIG_BLOCK_LIMIT
	Then increase the block count by FACTOR
*/
#define _BIG_BLOCK_LIMIT	8192
#define _BIG_BLOCK_FACTOR	15/10


size_t LONGLINE::CalcNewSize(size_t n)
{
	size_t m;


	if ((n>((size_t)size*_LONGLINE_BLOCKSIZE)) &&	// if this is an increase
		((size*_LONGLINE_BLOCKSIZE)>_BIG_BLOCK_LIMIT))	// and the size is already big enough
	{
		m=size*_BIG_BLOCK_FACTOR;	// use the factor

		if ((size_t)m*_LONGLINE_BLOCKSIZE<=(n+1))	// check if this is big enough
		{
			m=(n+1)/_LONGLINE_BLOCKSIZE;
			if (((n+1)%_LONGLINE_BLOCKSIZE)!=0)
				m++;
		}
	} else
	{
		m=(n+1)/_LONGLINE_BLOCKSIZE;
		if (((n+1)%_LONGLINE_BLOCKSIZE)!=0)
			m++;
	}

	return(m);
}


char *LONGLINE::operator+=(const char *src)
{
 return(Strcat(src));
}

char *LONGLINE::operator+=(const LONGLINE& p2)
{
 return(Strcat(p2));
}

char *LONGLINE::operator+=(char c)
{
 char st[2];
 st[0]=c;
 st[1]=0;
 return(operator+=(st));
}

char *LONGLINE::operator+=(long l)
{
 char st[128]; // more ???

 sprintf(st,"%ld",l);
 return(operator+=(st));
}

char *LONGLINE::operator=(const char *src)
{
 return(Strcpy(src));
}

char *LONGLINE::operator=(const LONGLINE& p2)
{
 return(Strcpy(p2));
}

LONGLINE LONGLINE::operator+(const char *src)
{
 LONGLINE t;

 t=*this;
 t+=src;

 return(t);
}

LONGLINE LONGLINE::operator+(const LONGLINE& p2)
{
 return(operator+(p2.p));
}

LONGLINE LONGLINE::operator+(char c)
{
 char st[2];

 st[0]=c;
 st[1]=0;

 return(operator+(st));
}

char LONGLINE::operator[](size_t s)
{
 return(p[s]);
}

int LONGLINE::operator==(const LONGLINE& p2) const
{
 return(!Strcmp(p2));
}

int LONGLINE::operator==(const char *p2) const
{
 return(!Strcmp(p2));
}

int LONGLINE::operator!=(const LONGLINE& p2) const
{
 return(Strcmp(p2));
}

int LONGLINE::operator!=(const char *p2) const
{
 return(Strcmp(p2));
}

void LONGLINE::Setsize(size_t n)
{
 Resize(CalcNewSize(n));
}

void LONGLINE::Set(size_t pos, char c)
{
 if (pos<len)
  p[pos]=c;
}

void LONGLINE::operator--()
{
 if (len!=0)
 {
  len--;
  p[len]=0;
  Resize(CalcNewSize(len));
 }
}

void LONGLINE::operator--(int n)
{
	operator--();
}


void LONGLINE::Lowercase()
{
 size_t n;
 char c;
 for (n=0;n<len;n++)
 {
  c=p[n];
  c=tolower(c);
  p[n]=c;
 }
}

void LONGLINE::Uppercase()
{
 size_t n;
 char c;
 for (n=0;n<len;n++)
 {
  c=p[n];
  c=toupper(c);
  p[n]=c;
 }
}

void	LONGLINE::ChopCRLF()
{
	while ((len>0) && ((p[len-1]=='\n') || (p[len-1]=='\r')))
	{
		len--;
		p[len]=0;
	}
	Resize(CalcNewSize(len));
}

void	LONGLINE::ChopSpaces()
{
	while ((len>0) && (isspace(p[len-1])))
	{
		len--;
		p[len]=0;
	}
	Resize(CalcNewSize(len));
}

/******************* NON memember functions *******************/

char *strcat(LONGLINE& p,const char *src)
{
 p.Strcat(src);

 return(p.p);
}

char *strcat(LONGLINE& p1,const LONGLINE& p2)
{
 p1.Strcat(p2);

 return(p1.p);
}

char *strcpy(LONGLINE& p,const char *src)
{
 p.Strcpy(src);

 return(p.p);
}

char *strcpy(LONGLINE& p1,const LONGLINE& p2)
{
 p1.Strcpy(p2);

 return(p1.p);
}

char *strncat(LONGLINE& p,const char *src, size_t s)
{
 p.Strncat(src,s);

 return(p.p);
}

char *strncat(LONGLINE& p1,const LONGLINE& p2, size_t s)
{
 p1.Strncat(p2,s);

 return(p1.p);
}

char *strncpy(LONGLINE& p,const char *src, size_t s)
{
 p.Strncpy(src,s);

 return(p.p);
}

char *strncpy(LONGLINE& p1,const LONGLINE& p2, size_t s)
{
 p1.Strncpy(p2,s);

 return(p1.p);
}

size_t strlen(const LONGLINE& p)
{
 return(p.len);
}

/**************************************************************/

void LONGMEM::Init()
{
 len=0;
 p=NULL;
 size=0;
 Resize(1);
}

LONGMEM::LONGMEM()
{
 Init();
}

LONGMEM::~LONGMEM()
{
	Resize(0);
}

LONGMEM::LONGMEM(const void *src, size_t s)
{
 Memcpy(src,s);
}

LONGMEM::LONGMEM(LONGMEM& p2)
{
 Memcpy(p2);
}

void *LONGMEM::Memcpy(const void *src, size_t s)
{
 Resize(CalcNewSize(s));
 memcpy(p,src,s);
 len=s;

 return(p);
}

void *LONGMEM::Memcpy(LONGMEM &p2)
{
 return(Memcpy(p2.p,(p2.size*_LONGMEM_BLOCKSIZE)));	// Posible error..changed by mistake.. don't remember previous state
}

void *LONGMEM::Memcpy(LONGMEM &p2, size_t s)
{
 return(Memcpy(p2.p,s));
}

void *LONGMEM::Memcat(const void *src, size_t s)
{
 size_t sum;

 sum=s+len;
 Resize(CalcNewSize(sum));
 memcpy((p+len),src,s);
 len=sum;

 return(p);
}

void *LONGMEM::Memcat(LONGMEM &p2)
{
 return(Memcat(p2.p,p2.len));
}

void *LONGMEM::Memcat(LONGMEM &p2, size_t s)
{
 return(Memcat(p2.p,s));
}

int LONGMEM::Memcmp(const void *src)
{
 return(Memcmp(src,len));
}

int LONGMEM::Memcmp(const void *src, size_t s)
{
 int n=0;

 if (s!=len)
  n=-1;
 else
  n=memcmp(p,src,s);

 return(n);
}

int LONGMEM::Memcmp(LONGMEM &p2)
{
 return(Memcmp(p2.p,p2.len));
}

LONGMEM LONGMEM::operator+(LONGMEM& p2)
{
 Memcat(p2);

 return(*this);
}

LONGMEM LONGMEM::operator+(char c)
{
 Memcat(&c,1);

 return(*this);
}

int LONGMEM::operator==(LONGMEM& p2)
{
 return(!Memcmp(p2));
}

int LONGMEM::operator==(const void *src)
{
 return(!Memcmp(src));
}

void *LONGMEM::operator+=(char c)
{
 return(Memcat(&c,1));
}

void *LONGMEM::operator+=(LONGMEM& p2)
{
 return(Memcat(p2));
}

void *LONGMEM::operator=(LONGMEM& p2)
{
 void *p;

 p=Memcpy(p2);
 return(p);
}

void LONGMEM::operator--()
{
 if (len!=0)
 {
  len--;
  Resize(CalcNewSize(len));
 }
}

void LONGMEM::Set(size_t pos, char c)
{
 if (pos>(size_t)(size*_LONGMEM_BLOCKSIZE)) Resize(CalcNewSize(pos));
 p[pos]=c;
}

void LONGMEM::Resize(long l)
{
 if (l!=size)
 {
  if (l==0)
  {
  	if (p!=NULL)
	{
		free(p);
		p=NULL;
	}
  } else
  if (p!=NULL)
   p=(char *)realloc(p,(size_t)l*_LONGMEM_BLOCKSIZE);
  else
   p=(char *)malloc((size_t)l*_LONGMEM_BLOCKSIZE);
  size=l;
  len=MIN(len,((size_t)l*_LONGMEM_BLOCKSIZE));
 }
}

long LONGMEM::CalcNewSize(size_t n)
{
 long m;

 m=(n+1)/_LONGMEM_BLOCKSIZE;
 if (((n+1)%_LONGMEM_BLOCKSIZE)!=0)
  m++;

 return(m);
}

void LONGMEM::Setsize(size_t n)
{
 Resize(CalcNewSize(n));
 len=n;
}

void LONGMEM::Empty()
{
 Resize(0);
}

char &LONGMEM::operator[](size_t pos)
{
	if (pos>(size_t)(size*_LONGMEM_BLOCKSIZE)) Resize(CalcNewSize(pos));
	if (len<pos+1) len=pos+1;
	return(p[pos]);
}

/******************* NON memember functions *******************/

void *memcpy(LONGMEM& p1,const void *p2, size_t s)
{
 return(p1.Memcpy(p2,s));
}

void *memcpy(LONGMEM& p1,LONGMEM& p2)
{
 return(p1.Memcpy(p2));
}

void *memcpy(LONGMEM& p1,LONGMEM& p2, size_t s)
{
 return(p1.Memcpy(p2,s));
}

void *memcat(LONGMEM& p1,const void *p2, size_t s)
{
 return(p1.Memcat(p2,s));
}

void *memcat(LONGMEM& p1,LONGMEM& p2)
{
 return(p1.Memcat(p2));
}

void *memcat(LONGMEM& p1,LONGMEM& p2, size_t s)
{
 return(p1.Memcat(p2,s));
}

int memcmp(LONGMEM& p1,LONGMEM& p2)
{
 return(p1.Memcmp(p2));
}

int memcmp(LONGMEM& p1,const void *p2, size_t s)
{
 return(p1.Memcmp(p2,s));
}

int memcmp(const void *p1,LONGMEM& p2, size_t s)
{
 return(p2.Memcmp(p1,s));
}


/**************************************************************/

#endif
