/****************************************/
/*  MEMPOOL class                       */
/****************************************/
/* Start      : 17/5/99                 */
/* Program ID :                         */
/* Version    : 1.00                    */
/* End        :                         */
/* Last Update: 17/5/99                 */
/* Comments   : Pool of same objects    */
/*                                      */
/****************************************/

#include "mempool.h"
#include "v/mem/dlist3.cc"

MEM_SECTOR::MEM_SECTOR()
{
 uses=0;
 p=NULL;
}

MEM_SECTOR::~MEM_SECTOR()
{
}

MEM_SECTOR::MEM_SECTOR(MEM_SECTOR &d)
{
 ddd(d);
}

MEM_SECTOR MEM_SECTOR::operator=(MEM_SECTOR &d)
{
 ddd(d);
 return (*this);
}

void MEM_SECTOR::ddd(MEM_SECTOR &d)
{
 uses=d.uses;
 p=d.p;
}

template <class T> MEMPOOL<T>::MEMPOOL()
{
 UsedN=FreeN=0;
}

template <class T> MEMPOOL<T>::~MEMPOOL()
{
}

template <class T> MEMPOOL<T>::MEMPOOL(MEMPOOL<T> &p)
{
 ddd(p);
}

template <class T> MEMPOOL<T> MEMPOOL<T>::operator=(MEMPOOL<T> &p)
{
 ddd(p);
 
 return(*this);
}

template <class T> void MEMPOOL<T>::ddd(MEMPOOL &p)
{
 UsedN=p.UsedN;
 FreeN=p.FreeN;
 pointers=p.pointers;
 dUsed=p.dUsed;
 dFree=p.dFree;
}

template <class T> void MEMPOOL<T>::Init(long l)
{
 Segment=l/3; // This is a reasonable number, to keep memory unfragmented
 Allocate(l);
}

/*
  Allocate l more blocks
 */
template <class T> void MEMPOOL<T>::Allocate(long l)
{
 long n,m;
 MEM_SECTOR *ms,ps;
 T *p2;

 m=sizeof(T)*l;
 p2=(T *)malloc(m);  // Allocate the memory
 ps.p=p2;
 pointers.PushL(ps);
 
 FreeN+=l;
 
 for (n=0;n<l;n++)
 {
  ms=malloc(sizeof(MEM_SECTOR));  // Allocate a MEM_SECTOR
  ms->p=&(p2[n]);
  dFree.PushL(ms);  // Set this as free
 }
}

template <class T> T *MEMPOOL<T>::Get()
{
 T *p;
 MEM_SECTOR *ms;

 if (FreeN==0)
  Allocate(Segment);
 
 ms=dFree.PopL();  /* Start using the last, in case the first went in the swap (because they are allocated first)*/
 FreeN--;
 p=ms->p;
 ms->uses++;
 dUsed.PushL(ms);
 UsedN++;
  
 return(p);
}

template <class T> void MEMPOOL<T>::Free(T *p)
{
 MEM_SECTOR *ms;
 int OK=0;

 dUsed.Rewind();
 
 while (!OK)
 {
  ms=dUsed.Get_Next();
  if (ms!=NULL)
  {
   if (ms->p==p) // We got it
   {
    dFree.PushL(ms);  // THIS SHOULD BE BEFORE REMOVECURRENT2 !!!!
    dUsed.RemoveCurrent2();
    UsedN--;
    FreeN++;
    OK=1;
   }
  } else
   OK=-1; /* We may have some problems!!! */
 }
}


template <class T> void MEMPOOL<T>::Show_Stats()
{
 printf("MEMPOOL: UsedN, FreeN == %ld, %ld\n",UsedN,FreeN);
}


