/****************************************/
/*  KEYTABLE class (header)             */
/****************************************/
/* Start      : 5/1/2000                */
/* Program ID :                         */
/* Version    : 2.03                    */
/* End        :                         */
/* Last Update: 3/7/2001  27/10/98      */
/* Comments   : Table with string keys  */
/*              (dynamic table)         */
/****************************************/

#ifndef _VLIB_V_MISC_KEYTABLE2_H
#define _VLIB_V_MISC_KEYTABLE2_H

#include "v/mem/dlist3.h"
#include "v/mem/longline.h"

#define KEYTABLE KEYTABLE2

/*
	TODO:			KEYTABLE2::lastsearch (WTF is this for ?)
	TODO:			AddSorted should do sorted insertion

	v2.03 3/7/2001: 	Bug fix. There was no check in AddSorted for an existing entry. Two entries could
				exist with the same key.
				Change: Just like KEYTABLE v1, now Add will change the value of an existing entry.
				Add: Case sensitivity: KEYTABLE2 it is not case sensitive (by default).
				to change this, define _KT_CASE_SENSITIVE
				NOTE: Case sensitivity should be defined once per class.. so this may change
	v2.02 29/4/2001:	Bug fix. GetNextBuf was a mistake. When passing two or more parameter, using [], the
				same parameter will be passed..
	v2.01 19/4/2001:	Bug fix. When deleting an entry, ibprev,ibnext where not updated.

	v2.00 15/1/2000:	Complete rewrite!!!!
				Now, keytable uses a complex hash table, structured as a tree.
				KEYTABLE template takes 2 more parameters. The Number of childs a
				tree node may have and the depth of the tree.
				e.x: KEYTABLE<LONGLINE,3,4> will create a keytable, that has a hash
				table of depth 4 (max) and each node has 3 childs. Nodes are created
				when needed. this way, on the 1st level there will be 3 nodes. On the
				2nd 3*3=9, on the 3rd 9*3=27 and on the 4th 3*27=81, which make a
				total of 1+3+9+27+81=121 (the 1 is the head). Those are the maximum
				numbers and not the actual. There will be created as many nodes as needed
				which is usualy less than the max.
				Data are kept in a sorted list. Insertions are slow (for now ??).

				The warning, described bellow does not exist, but someone should not
				rely on it.

	v1.01 6/1/2000:		when searching for an element, first check to see if the current element
				is the wanted one. This helps **A LOT** in the case decsribed bellow
				( I hope this will not create any bugs :)

	(30/12/99) WARNING!!!

	LONGLINE	*p;
	KEYTABLE<MYVAR> kt;
	MYVAR		*mv;

	kt.Rewind();
	while ((p=kt.Get_Next())!=NULL)
	{
		mv=kt[*p];	// This works.. BUT...
				// when using operator[], the Get_Next() position is lost.
				// In this case, there will be no problem, because it will
				// come back to kt[*p] (where it was).
				// This means, that this is the *ONLY* way to expect correct
				// results from a Rewind-Get_Next-operator[] combination
	}
*/

/*template <class T> struct KTOBJ // KEYTABLE Object
{
 long ID;
 LONGLINE str;
 T d;
 private:
  void vvv(KTOBJ& p)
  {
   ID=p.ID;
   str=p.str;
   d=p.d;
  }
 public:
  KTOBJ() {}
  KTOBJ(KTOBJ& p)
  {
   vvv(p);
  }
  KTOBJ operator=(KTOBJ &p)
  {
   vvv(p);
   return(*this);
  }
};*/
template <class T,int HS> class KT_HASHBLOCK;
template <class T,int HS> struct KTOBJ2	// KEYTABLE Object
{
	long			ID;	// Is this needed?
	LONGLINE		str;
	T			d;
	KTOBJ2<T,HS>		*prev;
	KTOBJ2<T,HS>		*next;
	KTOBJ2<T,HS>		*ibnext,*ibprev;	// In block/node next/prev
	KT_HASHBLOCK<T,HS>	*hblock;
 public:
 	KTOBJ2()
 	{

 		ibprev=ibnext=prev=next=NULL;
 		hblock=NULL;
 		ID=-1;
 	}
};

/*
	T : Type
	HS: Hash table size (1-256)
*/
template <class T,int HS> class KT_HASHBLOCK
{
 public:
	KT_HASHBLOCK<T,HS>	*parent;
	KT_HASHBLOCK<T,HS>	*childs[HS];
	KTOBJ2<T,HS>		*p;
	long			level;
 public:
 	KT_HASHBLOCK()
 	{
 		int	n;
 		parent=NULL;
 		for (n=0;n<HS;n++) childs[n]=NULL;
 		p=NULL;
 		level=-1;
 	}
 	int NoChilds()
 	{
 		int	n,c=1;
 		for (n=0;(n<HS && c==1);n++)
 		{
 			if (childs[n]!=NULL) c=0;
 		}
 		return(c);
 	}
 	void UnlinkChild(KT_HASHBLOCK<T,HS> *p)
 	{
 		int	n,OK=0;
 		for (n=0;(n<HS && !OK);n++)
 		{
 			if (childs[n]==p)
 			{
 				childs[n]=NULL;
 				OK=1;
 			}
 		}
 	}
};

/*
 The data are copied.
 HS: Hash table size
 MD: Max depth
 */
template <class T,int HS=256,int MD=1> class KEYTABLE2
{
 private:
  long			NextID;
  KTOBJ2<T,HS>		*lastsearch,*lastGetNext;	// lastGetNext is used for Get_Next();
  KT_HASHBLOCK<T,HS>	hash_head;
  KT_HASHBLOCK<T,HS>	*lasthash;	// GetHashNode sets this.. It is the closest node to the needed
  int			hbs;		// hbs== The number of characters, assigned to each hash node hbs*HS==256
//  KTOBJ<T,HS>		*head;
  KTOBJ2<T,HS>		*head,*tail;
// The following removed at 29/4/2001
//  LONGLINE		GetNextBuf;	// Contains the value of the last GetNext executed. GetNext returns this address
 public:
//  DL_List< KTOBJ<T> > d;
  int			count;
 private:
  int isbefore(char *st1, char *st2);	// return: 1: if st2<st1. (shorting order)
  KT_HASHBLOCK<T,HS> *GetHashNode(const char *name);
  //, KT_HASHBLOCK<T,HS> *cur=&hash_head, int curdep=0);
  void HashAdd(KTOBJ2<T,HS> *d);	// Add a hash node , pointing to d if it doesn't exist, or make the apropriate adjustments if it exist
  void HashDel(KTOBJ2<T,HS> *d);	// Prepere tha hash table, for the target node to be deleted. Adjust whatever is needed
  void DelTree(KT_HASHBLOCK<T,HS> *tree);	// Remove a tree and it's childs
  void CheckAndDel(KT_HASHBLOCK<T,HS> *node);
  long Exist(const LONGLINE &s);
  KTOBJ2<T,HS> *GetKTO(const LONGLINE& s);
  void vvv(KEYTABLE2& p);
  long AddSorted(const LONGLINE& str, T p);
  void ClearAll();
  void Init();
  int isinsameblock(const char *st1, const char *st2);	// return 1 if st1 and st2 are in the same block

//  void ClearTable();
 public:
  KEYTABLE2();
  ~KEYTABLE2();
  KEYTABLE2(KEYTABLE2<T,HS,MD> &p);
  KEYTABLE2 operator=(KEYTABLE2<T,HS,MD>& p);
  long Add(const LONGLINE& str, T p);
  T *Get(const LONGLINE &str);
  void Del(const LONGLINE &str);
  T *operator[](const LONGLINE &str);
  T *operator[](const char *s);

  void Rewind();
  LONGLINE *Get_Next();
  void ClearTable();
};

#endif


/*
 Table (array) of objects, using a string as a key.
 */
