/****************************************/
/*  Dynamic Library Loading             */
/****************************************/
/* Start      : 20/4/2001               */
/* Program ID :                         */
/* Version    : 1.00                    */
/* End        :                         */
/* Last Update: 20/4/2001               */
/* Comments   :                         */
/*                                      */
/****************************************/

#ifndef _VLIB_V_MISC_LIBLOADER_H
#define _VLIB_V_MISC_LIBLOADER_H

#include "dlfcn.h"
#include "v/mem/dlist3.h"
#include "v/mem/longline.h"
#include "v/misc/keytable2.h"

#define LOADLIB_ERROR	0x00		// Error while loading shared library
#define LOADLIB_NOINIT	0x01		// No initialize function in library
#define LOADLIB_OK	0x10		// All ok
#define UNLOADLIB_ERROR	LOADLIB_ERROR	// Error when trying to unload lib
#define UNLOADLIB_OK	LOADLIB_OK	// Ok

#define GETFUNC_OK	0x01		// Fucntion found
#define GETFUNC_ERROR	0x02		// Error


class LIBLOADER
{
 private:
 public:
	struct LIB
	{
		void		*handle;
		char		*name;
	};

 	DL_List<LIB *>		Libs;		// The opened Libraries

	enum sym_resolve { lazy=0, now=1 };
 private:
//	void	ddd(LIBLOADER &d);
	void	DelLib(const char *libname);
 public:
			LIBLOADER();
			~LIBLOADER();			// This will not dlclose any lib.
//			LIBLOADER(LIBLOADER &d);	// There is no sense in copying this class. This will cause more problems
//	LIBLOADER	operator=(LIBLOADER &d);	// than it will solve

	int	loadlib(const char *libname, sym_resolve resolv=now);	// dlopen a lib
	int	unloadlib(const char *libname);	// dlclose a lib
	int	unloadlib(LIB *plib);
	LIB	*GetLib(const char *libname);
};

class LIBFUNCLIST
{
 private:
 public:
	struct	LIBFUNC
	{
		void		*func;
		char		*funcname;
		LIBLOADER::LIB	*lib;
	};

//	typedef DL_List<LIBFUNC *>	LIBFUNCLIST;
	typedef KEYTABLE<LIBFUNC *>	kLIBFUNCLIST;

	kLIBFUNCLIST		funcs;

 private:
	void		ddd(LIBFUNCLIST &d);
	LIBFUNC		*lfgetfunc(const char *funcname);
 public:
			LIBFUNCLIST();
			~LIBFUNCLIST();
			LIBFUNCLIST(LIBFUNCLIST &d);
	LIBFUNCLIST	operator=(LIBFUNCLIST &d);

	int		addfunc(const char *funcname, LIBLOADER::LIB *plib);
	int		delfunc(const char *funcname);
	void		Clear();
	void		*getfunc(const char *funcname);
	int		exist(const char *funcname);
};

#endif
