/****************************************/
/*  Logging facility                    */
/****************************************/
/* Start      : 11/10/98                */
/* Program ID :                         */
/* Version    : 1.03                    */
/* End        :                         */
/* Last Update: 19/05/2002              */
/* Comments   :                         */
/*                                      */
/****************************************/

#include "log.h"
#include "v/misc/keytable2.cc"
//#include "v/vtime.cc"

/*
  	19/5/2002  v/1.03	Rename log to vlog - It was a bug when passing only one argument
	5/3/2002   v1.02:	Minor bug fix - don't core on invalid loglevels
	2/2/2002   v1.01:	Added Log() with va_list as argument
*/

LOGGER::LOGGER()
{
}

LOGGER::~LOGGER()
{
 ClearAll();
}

void LOGGER::OpenLog(LOGTABLE &p)
{
 char *fn;
 LOGFNAME *lfn;

 if (!p.log)  // If there was no open log
 {
  p.log=1;
  fn=p.fn;
  lfn=tf.Get(fn);
  if (lfn==NULL) // And there is no entry for this filename
  {
   lfn=new(LOGFNAME);
   lfn->Count++; // Then increase the counter
   if (!strcmp(fn,"STDOUT"))  // check for special
    lfn->f.fp=stdout;
   else if (!strcmp(fn,"STDERR")) // check for special
    lfn->f.fp=stderr;
   else
   {
    lfn->f.Open(fn);   // Open the file for writing
    lfn->f.Seekend();  // And go to the end of file
   }
   tf.Add(fn,*lfn);
   delete(lfn);
  } else
   lfn->Count++; // Else, just increase the counter

 }
}

void LOGGER::CloseLog(LOGTABLE &p)
{
 char *fn;
 LOGFNAME *lfn;

 if (p.log) // If the log was open
 {
  p.log=0;
  fn=p.fn;
  lfn=tf.Get(fn);
  if (lfn!=NULL)	// 28/7/2002 -- avoid core
  {
   lfn->Count--;  // Decrease the counter
   if (!lfn->Count) // If count==0
   {
    if ((strcmp(fn,"STDOUT")) || (strcmp(fn,"STDERR"))) // check for special
     lfn->f.Close();  // Then close the file
    tf.Del(fn);   // And delete the entry
   }
  }
 }
}

void LOGGER::AddLevel(const char *levelname,const  char *filename, int log)
{
 LOGTABLE p;
 
 strcpy(p.fn,filename);
 p.log=0;
 if (log)
  OpenLog(p);
 t.Add(levelname,p);
}

void LOGGER::AddLevel(const char *levelname,const  char *filename)
{
 AddLevel(levelname,filename,1);
}

void LOGGER::DelLevel(const char *levelname)
{
 Del(levelname);
}

void LOGGER::ClearAll()
{
 LONGLINE *p;
 LOGTABLE *d;
// LOGFNAME *lfn;

 t.Rewind();
 while ((p=t.Get_Next())!=NULL)
 {
  d=t.Get(*p);
  CloseLog(*d);
  t.Del(*p);
  t.Rewind();
 }
}

void LOGGER::plog(const char *levelname,const  char *format, va_list l )
{
 LOGTABLE *p;
 LOGFNAME *pf;
 int n;
 
 p=t.Get(levelname);
 if (p==NULL) return;	// 5/3/2002 -- avoid core
 if (p->log)
 {
  pf=tf.Get(p->fn);
  if (pf==NULL) return; // 28/7/2002 -- avoid core
  if (LogType & LOGTYPE_TIME)
  {
//   fprintf(pf->f.fp,vtime.StrTime());
   fprintf(pf->f.fp,vtime.str());
   fputs(": ",pf->f.fp);
  }
  if (LogType & LOGTYPE_CLASS)
  {
   fprintf(pf->f.fp,levelname);
   fputs(": ",pf->f.fp);
  }
  vfprintf(pf->f.fp,format,l);
  if (LogType & LOGTYPE_ENTER)
  {
   n=strlen(format);
   if ((format[n-1]!='\n') && !((format[n-1]=='\r') && (format[n-2]=='\n')))
    fputc('\n',pf->f.fp);
  }
  fflush(pf->f.fp);
 }
}

void LOGGER::Log(const char *levelname,const  char *format, ... )
{
 va_list l;

 va_start(l,format);
 
 plog(levelname,format,l);
 
 va_end(l);
}

void LOGGER::log(const char *levelname, const char *format, ... )
{
 va_list l;

 va_start(l,format);

 plog(levelname,format,l);
 
 va_end(l);
}

void LOGGER::vlog(const char *levelname,const char *format, va_list l)
{
	plog(levelname,format,l);
}

void LOGGER::vLog(const char *levelname,const char *format, va_list l)
{
	plog(levelname,format,l);
}

int LOGGER::SetLog(const char *levelname,int log)
{
 LOGTABLE *p;
 int ret;
 p=t.Get(levelname);
 if (p==NULL) ret=LOGSET_FAIL;
 else
 {
  if (log!=p->log)
  {
//   p->log=log;
//   if (p->log)
   if (log)
    OpenLog(*p);
   else
    CloseLog(*p);
  }
  ret=LOGSET_DONE;
 }
 return(ret);
}

void LOGGER::Del(const char *levelname)
{
// LOGFNAME *lfn;
 LOGTABLE *lt;

 lt=t.Get(levelname);
 if (lt!=NULL)
 {
  CloseLog(*lt);
  t.Del(levelname);
 }
}
