/****************************************/
/*  Logging facility (header)           */
/****************************************/
/* Start      : 11/10/98                */
/* Program ID :                         */
/* Version    : 1.02                    */
/* End        :                         */
/* Last Update: 19/05/2002              */
/* Comments   :                         */
/*                                      */
/****************************************/

#ifndef _VLIB_V_MISC_LOG_H
#define _VLIB_V_MISC_LOG_H

/*
 class LOGGER:
  Open an output file
  Log to file
  Support log levels
  
  */

#include "stdio.h"
#include "stdarg.h"
#include "v/misc/keytable2.h"
#include "v/file/filebase.h"
#include "string.h"
#include "v/vtime.h"

#define _LOG_MAX_FNAME 512

struct LOGTABLE
{
 char fn[_LOG_MAX_FNAME];
 // FILEBASE f;
 int log;

 private:
  void vvv(const LOGTABLE& p)
  {
   strncpy(fn,p.fn,_LOG_MAX_FNAME);
   log=p.log;
  }
 public:
  LOGTABLE() {}
  LOGTABLE(const LOGTABLE&p)
  {
   vvv(p);
  }
  LOGTABLE operator=(const LOGTABLE &p)
  {
   //   LOGTABLE::LOGTABLE(p);
   vvv(p);
   return(*this);
  }
};

struct LOGFNAME
{
 FILEBASE f;
 long Count;  // Number of LOGTABLE entries, that are using this fname

 private:
  void vvv(const LOGFNAME& p)
  {
   f=p.f;
   Count=p.Count;
  }
 public:
  LOGFNAME() {}
  LOGFNAME(const LOGFNAME& p)
  {
   vvv(p);
  }
  LOGFNAME operator=(const LOGFNAME &p)
  {
   //   LOGFNAME::LOGFNAME(p);
   vvv(p);
   return(*this);
  }
};

/*
 There are many LOGTABLE entries. Each of them is a logging type.
 Each of them, may be logges or not. Each of them have a filename.

 More than one LOGTABLE entries may have the same filename. In this case,
 the same file is used. To make this possible, another KEYTABLE is used,
 containing the FILEBASE entries, using the filename as the key.
 */

#define LOGTYPE_RAW    1   // Log only what is provided
#define LOGTYPE_TIME   2   // Also log current time
#define LOGTYPE_ENTER  4   // Make sure, there is a CR/LF at eol
#define LOGTYPE_CLASS  8   // Also log the class

#define LOGSET_DONE 1	// LOGGER::Set succeeded
#define LOGSET_FAIL 0   // LOGGER::Set failed (no such log?)

class LOGGER
{
 public:
  KEYTABLE<LOGTABLE> t;
  KEYTABLE<LOGFNAME> tf;
  VTIME vtime;
 public:
  int LogType;
 private:
  void ClearAll();
  void plog(const char *levelname,const  char *format, va_list l);
  void OpenLog(LOGTABLE &p);
  void CloseLog(LOGTABLE &p);
  void Del(const char *levelname);
 public:
  LOGGER();
  ~LOGGER();
  void AddLevel(const char *levelname,const  char *filename, int log);
  void AddLevel(const char *levelname,const  char *filename);
  int SetLog(const char *levelname, int log);
  void DelLevel(const char *levelname);
  void Log(const char *levelname,const  char *format, ... );
  void log(const char *levelname,const  char *format, ... );
  void vLog(const char *levelname,const  char *format, va_list l);
  void vlog(const char *levelname,const  char *format, va_list l);
};

#endif
