/****************************************/
/* Logging facility (2nd version)	*/
/****************************************/
/* Start      : 14/04/2004		*/
/* Version    : 2.00			*/
/* Last Update: 14/04/2004		*/
/****************************************/

#ifndef _VLIB_V_MISC_LOG2_H
#define _VLIB_V_MISC_LOG2_H

#include "log2.h"

#include "v/vtime.h"

#include <v/misc/keytable2.h>

#include <stdarg.h>
#include <syslog.h>

#define LOGTYPE_TIME	2
#define LOGTYPE_ENTER	4
#define LOGTYPE_CLASS	8

/*
 * Logging facility v2.
 *
 * Supports string and integer loglevels.
 *
 * Only logs to a single file (or syslog).
 *
 * Priority 0 is the highest priority. To conform with syslog priorities
 * I suggest priority values 0-7 which will be mapped to 
 * 0: LOG_EMERG, 1: LOG_ALERT, 2: LOG_CRIT, 3: LOG_ERR, 4: LOG_WARNING,
 * 5: LOG_NOTICE, 6: LOG_INFO, 7: LOG_DEBUG
 *
 * values >7 will be mapped to LOG_DEBUG too
 * values <0 will be mapped to LOG_EMERG
 */
class LOGGER2
{
 private:
	struct LOGLEVEL
	{
		LONGLINE	name;	// the name of the loglevel
		int		on;	// 1 if it is on
	};

	enum	LogFacility
	{
		LogNone, LogFile, LogStdout, LogStderr, LogSyslog
	};

	static int		syslogmap[8];		// syslog levels mapping
	static int		syslog_count;		// number of instances that use syslog

	KEYTABLE<LOGLEVEL>	loglevels;

	int			minlevel;		// the minimum logging level
	LogFacility		facility;

	LONGLINE		fn;
	FILE			*f;

	int			logtype;		// how to log
	VTIME			vtime;
 public:
 private:
	void	ddd		(LOGGER2 &d);

	int	OpenFile	(const char *fn);
	void	CloseFile	();			// close logger file (if any);

	void	plogFile	(const char *prefix, const char *format, va_list l);
	void	plog		(int level, const char *format, va_list l);
	void	plog		(const char *levelstr, const char *format, va_list l);

	int	checklevel	(const char *level);
 public:
		LOGGER2		();
		~LOGGER2	();
		LOGGER2		(LOGGER2 &d);
	LOGGER2	operator=	(LOGGER2 &d);

	void	SetLevel	(int n);			// set the minimum logging level
	void	SetLevel	(const char *p, int on);	// set on/off for this level
	void	SetLevel	(const LONGLINE &ll, int on);

	void	AddLevel	(const char *name, int on=1);
	void	AddLevel	(const LONGLINE &ll, int on=1);
	void	DelLevel	(const char *name);
	void	DelLevel	(const LONGLINE &ll);

	int	SetFile		(const char *fn);	// Return: 1: OK, 0: Error
	void	SetStdout	();
	void	SetStderr	();
	void	SetSyslog	(const char *ident, int facility=LOG_USER);

	void	SetLogType	(int n);	// set logging flags

	void	CloseLog	();		// Stop logging

	void	Log		(int level, const char *format, ...);
	void	log		(int level, const char *format, ...);
	void	vLog		(int level, const char *format, va_list l); 
	void	vlog		(int level, const char *format, va_list l);

	void	Log		(const char *level, const char *format, ...);
	void	log		(const char *level, const char *format, ...);
	void	vLog		(const char *level, const char *format, va_list l);
	void	vlog		(const char *level, const char *format, va_list l);
};

#endif

