/****************************************/
/*					*/
/****************************************/
/* Start      : 10/03/2002		*/
/* Program ID : 			*/
/* Version    : 1.00			*/
/* End        : 			*/
/* Last Update: 24/01/2004		*/
/* Comments   : 			*/
/*                                      */
/****************************************/

#ifndef _VLIB_V_MISC_NKEYTABLE_CC
#define _VLIB_V_MISC_NKEYTABLE_CC

#include "nkeytable.h"

#include "string.h"

template <class T> void NKEYTABLE<T>::init_nkt()
{
	count=0;
	ZeroNode(&hash_head);
	hash_head.level=0;
	head=tail=lastGetNext=NULL;
}

template <class T> NKEYTABLE<T>::NKEYTABLE()
{
	init_nkt();
}

template <class T> NKEYTABLE<T>::~NKEYTABLE()
{
	ClearTable();
}

//#define	MK_KEYS(x,y)	x[0]=(y>>24) & 0xFF; x[1]=(y>>16) & 0xFF; x[2]=(y>>8) & 0xFF; x[3]=y & 0xFF;
#define	MK_KEYS(x,y)	x[0]=(y>>28) & 0xF; x[1]=(y>>24) & 0xF; x[2]=(y>>20) & 0xF; x[3]=(y>>16) & 0xF; \
			x[4]=(y>>12) & 0xF; x[5]=(y>>8) & 0xF; x[6]=(y>>4) & 0xF; x[7]=y & 0xF;
//#define CREATE_KEY(x,y)	y=(x[0]<<28) + (x[1]<<24) + (x[2]<<20) + (x[3]<<16) + (x[4]<<12) + (x[5]<<8) + (x[6]<<4) + x[7];

/*
	Erase any unused tree entries.
	the FNode must be already deleted.
*/
template <class T> void NKEYTABLE<T>::CheckAndDel(u_32 key)
{
	NK_NODE		*pnode[NK_DEPTH],*ppnode;
	u_8		k[NK_DEPTH];
	int		n;

	MK_KEYS(k,key);

	ppnode=&hash_head;
	for (n=0;n<NK_DEPTH;n++)
		pnode[n]=NULL;
	for (n=0;n<NK_DEPTH;n++)
	{
		pnode[n]=ppnode;;
		if (pnode[n]==NULL) n=NK_DEPTH;
		else ppnode=pnode[n]->childs.nodes[k[n]];
	}
	for (n=(NK_DEPTH-1);n>0;n--)
	{
		if (pnode[n]->count==0)	// if this one has no childs
		{
			delete(pnode[n]);	// delete it
			pnode[n-1]->childs.nodes[k[n-1]]=NULL;	// zero the apropriate pointer
			pnode[n-1]->count--;	// and decrease the count
		}
	}

}

template <class T> void NKEYTABLE<T>::ZeroNode(NK_NODE *node)
{
//	memset(node->childs.nodes,0,256*sizeof(NK_NODE *));	// dirty... bye sizeof(NK_NODE *)==sizeof(NK_FNODE *)
	memset(&(node->childs),0,sizeof(typename NK_NODE::u_childs));
	node->count=0;
}

template <class T> void NKEYTABLE<T>::ClearNode(NK_NODE *node)
{
	int	n;

	if (node==NULL) return;

	if (node->level<(NK_DEPTH-1))
	{
		NK_NODE		*pnode;

		for (n=0;n<NK_WIDTH;n++)	// for each child node
		{
			pnode=node->childs.nodes[n];
			if (pnode!=NULL)
			{
				ClearNode(pnode);	// delete its childs
				delete(pnode);		// and delete it
			}
		}
	} else
	{
		NK_FNODE	*pfnode;

		for (n=0;n<NK_WIDTH;n++)	// for each child (final) node
		{
			pfnode=node->childs.fnodes[n];
			if (pfnode!=NULL)
			{
//				delete(pfnode);	// delete it
				deleteFNode(pfnode);
			}
		}
	}

	ZeroNode(node);
}

// Clear all data
template <class T> void	NKEYTABLE<T>::ClearTable()
{
	ClearNode(&hash_head);
	count=0;
}

template <class T> typename NKEYTABLE<T>::NK_FNODE *NKEYTABLE<T>::MkFNode(u_32 key)
{
	NK_NODE		*pnode,*ppnode;
	NK_FNODE	*ret=NULL;
	u_8		k[NK_DEPTH];
	int		n;

	MK_KEYS(k,key);

	ppnode=&hash_head;
	for (n=0;n<(NK_DEPTH-1);n++)
	{
		pnode=ppnode->childs.nodes[k[n]];
		if (pnode==NULL)
		{
			ppnode->count++;	// VVV
			pnode=ppnode->childs.nodes[k[n]]=new NK_NODE;

			pnode->level=ppnode->level+1;
			ZeroNode(pnode);
		}
		ppnode=pnode;
	}

	ret=pnode->childs.fnodes[k[NK_DEPTH-1]];
	if (ret==NULL)
	{
//		ret=pnode->childs.fnodes[k[3]]=new NK_FNODE;
		pnode->count++;	// VVV
		ret=pnode->childs.fnodes[k[NK_DEPTH-1]]=newFNode();
		ret->key=key;
	}

	return(ret);
}

template <class T> void NKEYTABLE<T>::Add(u_32 key, T data)
{
	NK_FNODE	*pfnode;

	pfnode=MkFNode(key);
	pfnode->d=data;
}

template <class T> void NKEYTABLE<T>::Add(u_32 key, T* data)
{
	NK_FNODE	*pfnode;

	pfnode=MkFNode(key);
	pfnode->d=*data;
}

template <class T> typename NKEYTABLE<T>::NK_NODE *NKEYTABLE<T>::GetLastNode(u_32 key)
{
	NK_NODE		*pnode,*ppnode;
	u_8		k[NK_DEPTH];
	int		n;

	MK_KEYS(k,key);

	ppnode=&hash_head;
	for (n=0;n<(NK_DEPTH-1);n++)
	{
		pnode=ppnode->childs.nodes[k[n]];
		if (pnode==NULL) return(NULL);
		ppnode=pnode;
	}

	return(pnode);
}

template <class T> typename NKEYTABLE<T>::NK_FNODE *NKEYTABLE<T>::GetFNode(u_32 key)
{
	NK_NODE		*pnode;
	NK_FNODE	*ret=NULL;
	u_8		k[NK_DEPTH];

	MK_KEYS(k,key);

	pnode=GetLastNode(key);

	if (pnode==NULL) return(NULL);

	ret=pnode->childs.fnodes[k[NK_DEPTH-1]];

	return(ret);
}


template <class T> T *NKEYTABLE<T>::Get(u_32 key)
{
	NK_FNODE	*pfnode;

	pfnode=GetFNode(key);

	return (pfnode==NULL ? NULL : &pfnode->d);
}

template <class T> int NKEYTABLE<T>::Exist(u_32 key)
{
	NK_FNODE	*pfnode;

	pfnode=GetFNode(key);

	return (pfnode==NULL ? 0 : 1);
}

template <class T> void NKEYTABLE<T>::Del(u_32 key)
{
	NK_FNODE	*pfnode;
	NK_NODE		*pnode;
	u_8		k[NK_DEPTH];

	MK_KEYS(k,key);

	pnode=GetLastNode(key);

	if (pnode==NULL) return;

	pfnode=pnode->childs.fnodes[k[NK_DEPTH-1]];
	if (pfnode==NULL) return;

	pnode->childs.fnodes[k[NK_DEPTH-1]]=NULL;
	pnode->count--;	//VVV
//	delete(pfnode);
	deleteFNode(pfnode);
	CheckAndDel(key);
}

template <class T> typename NKEYTABLE<T>::NK_FNODE *NKEYTABLE<T>::newFNode()
{
	NK_FNODE	*p;

	p=new NK_FNODE;

	count++;

	p->prev=tail;
	p->next=NULL;
	if (tail!=NULL)
		tail->next=p;
	tail=p;

	if (head==NULL) head=tail;

	return(p);
}

template <class T> void NKEYTABLE<T>::deleteFNode(NK_FNODE *fnode)
{
	if (fnode->prev==NULL)
		head=fnode->next;
	else
		fnode->prev->next=fnode->next;

	if (fnode->next==NULL)
		tail=fnode->prev;
	else
		fnode->next->prev=fnode->prev;

	if (lastGetNext==fnode)
		lastGetNext=fnode->next;

	count--;

	delete(fnode);
}

template <class T> void NKEYTABLE<T>::Rewind()
{
	lastGetNext=head;
}

template <class T> u_32 NKEYTABLE<T>::Get_Next()
{
	u_32	ret;

	if (lastGetNext==NULL)
		return(NK_INVALID);

	ret=lastGetNext->key;

	lastGetNext=lastGetNext->next;

	return(ret);
}

template <class T> T *NKEYTABLE<T>::operator[](u_32 key)
{
	return(Get(key));
}

template <class T> void NKEYTABLE<T>::ddd(NKEYTABLE<T> &d)
{
	u_32	n;

	d.Rewind();
	while ((n=d.Get_Next())!=NK_INVALID)
	{
		Add(n,d[n]);
	}
}

template <class T> NKEYTABLE<T>::NKEYTABLE(NKEYTABLE<T> &d)
{
	init_nkt();
	ddd(d);
}

template <class T> NKEYTABLE<T> NKEYTABLE<T>::operator=(NKEYTABLE<T> &d)
{
	ClearTable();

	ddd(d);

	return(d);
}

#endif

