/****************************************/
/*					*/
/****************************************/
/* Start      : 04/05/2002		*/
/* Program ID : 			*/
/* Version    : 1.00			*/
/* End        : 			*/
/* Last Update: 04/05/2002		*/
/* Comments   : 			*/
/*                                      */
/****************************************/

#include "pidfile.h"

#include "v/lconfig.h"

#include "v/mem/longline.h"

#ifdef FLOCK_NEEDS_BSD_COMPAT
#	define _BSD_COMPAT
#endif

#include <errno.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

/*
	Pidfile and locking.
	There is support for only 1 pidfile
*/

int	vlib_pidfilefd=-1;
char	*vlib_pidfile=NULL;


/*
 * Check if the errno indicates a permission problem
 *
 * Return 1/0: Yes/no
 */
int	pidfile_errno_is_perm()
{
	if (errno==EISDIR || errno==EACCES || errno==EROFS ||
		errno==ELOOP || errno==EMFILE || errno==ENFILE)
		return(1);

	return(0);
}

/*
 * Check if a pidfile is locked.
 *
 * Return:
 *	0:	No
 *	1:	Permissions problem
 *	>1:	pid
*/
int	pidfile_islocked(const char *pidfile)
{
	struct flock	fl;
	int		fd,n;

	fl.l_type=F_WRLCK;
	fl.l_whence=SEEK_SET;
	fl.l_start=0;
	fl.l_len=0;
	fl.l_pid=0;

	fd=open(pidfile,O_WRONLY);
	if (fd==-1)
	{
		if (pidfile_errno_is_perm())
			return(1);
		else
			return(0);
	}

	n=fcntl(fd,F_GETLK, &fl);
	close(fd);

	if ((n==-1) || (fl.l_type==F_UNLCK))
		return(0);

	return(fl.l_pid);
}

/*
 * Do the pidfile locking.
 *
 * Return:
 *	-1: Race condition
 *	0: No lock existed. File was updated/created
 *	1: Permissions problem
 *	>1: File existed and it was locked - pid is returned
 */
int	pidfile_dolock(const char *pidfile)
{
	struct flock	fl;
	int		fd,n;
	int		ret=0;

	vlib_pidfilefd=-1;

	fl.l_type=F_WRLCK;
	fl.l_whence=SEEK_SET;
	fl.l_start=0;
	fl.l_len=0;
	fl.l_pid=getpid();

	fd=open(pidfile,O_RDWR);
	if (fd>=0)	// file exists
	{
		n=fcntl(fd, F_SETLK, &fl);
		if (n==0)	// wasn't locked
		{
			fl.l_type=F_UNLCK;
			fcntl(fd,F_SETLK,&fl);	// unlock it
			close(fd);
			vlib_pidfilefd=open(pidfile,O_RDWR | O_TRUNC | O_SYNC);
			ret=0;
		} else	// locked
		{
			n=fcntl(fd, F_GETLK, &fl);
			if (n==-1)	// race condition ?
			{
				close(fd);
				vlib_pidfilefd=open(pidfile, O_RDWR | O_CREAT | O_SYNC, S_IRUSR | S_IWUSR);
				ret=0;
			} else
			{
				ret=fl.l_pid;
			}
		}
	} else if (pidfile_errno_is_perm())
	{
		ret=1;
	} else
	{
		vlib_pidfilefd=open(pidfile, O_RDWR | O_CREAT | O_SYNC, S_IRUSR | S_IWUSR);
		ret=0;
	}

	if (ret==0)	/* At this point, if ret==0, file is open with size 0 and ready for writting */
	{
		char	buf[30];

		sprintf(buf,"%d\n",getpid());
		write(vlib_pidfilefd,buf,strlen(buf));       /* Write the pid */

//		n=flock(vlib_pidfilefd,LOCK_EX);	// and lock it all
		fl.l_pid=getpid();
		fl.l_type=F_WRLCK;
		fl.l_start=0;
		fl.l_len=0;
		fl.l_whence=SEEK_SET;
		n=fcntl(vlib_pidfilefd, F_SETLK, &fl);

		if (vlib_pidfile!=NULL) free(vlib_pidfile);
		vlib_pidfile=strdup(pidfile);
	}

	return(ret);
}

/*
        Remove the advisory lock and the pidfile
*/
void    pidfile_dounlock()
{
	int		n;
	struct flock	fl;

	fl.l_type=F_UNLCK;
	fl.l_whence=SEEK_SET;
	fl.l_start=0;
	fl.l_len=0;
	fl.l_pid=getpid();

	if (vlib_pidfilefd==-1)
		return;

	n=fcntl(vlib_pidfilefd,F_SETLK,&fl);
        close(vlib_pidfilefd);
        unlink(vlib_pidfile);
	free(vlib_pidfile);
	vlib_pidfile=NULL;
}

