/****************************************/
/*					*/
/****************************************/
/* Start      : 04/05/2002		*/
/* Program ID : 			*/
/* Version    : 1.00			*/
/* End        : 			*/
/* Last Update: 04/05/2002		*/
/* Comments   : 			*/
/*                                      */
/****************************************/

#include "process.h"

#include "sys/time.h"
#include "sys/types.h"
#include "sys/stat.h"
#include "sys/resource.h"
#include "sys/ioctl.h"
#include "unistd.h"
#include "fcntl.h"
#include "signal.h"
#include "stdlib.h"

void	disassociate_terminal()
{
#ifdef BSD
	int	fd;


	fd=open("/dev/tty", O_RDWR);
	if (fd>=0)
	{
		ioctl(fd, TIOCNOTTY, NULL);
		close(fd);
	}
	setpgrp(0, getpid());
#else
	setsid();
#endif
}

void	close_all_fds(int all)
{
	int	fd;
	int	fd_max;
	int	fd_min;

	if (all) fd_min=0; else fd_min=3;

	// Get the current limit
	fd_max=get_fdslimit();

	// And close every possible open fd
	for (fd=fd_min;fd<fd_max;fd++)
		close(fd);
}

void	close_all_fds_except_one(int exfd, int all)
{
	int	fd;
	int	fd_max;
	int	fd_min;

	if (all) fd_min=0; else fd_min=3;

	// Get the current limit
	fd_max=get_fdslimit();

	// And close every possible open fd
	for (fd=fd_min;fd<fd_max;fd++)
	{
		if (fd!=exfd)
			close(fd);
	}
}

long	get_fdslimit()
{
	rlimit	rl;
	int	n;

	n=getrlimit(RLIMIT_NOFILE,&rl);
	if (n==-1) return(-1);

	return((long)rl.rlim_cur);
}

long	get_fdhlimit()
{
	rlimit	rl;
	int	n;

	n=getrlimit(RLIMIT_NOFILE,&rl);
	if (n==-1) return(-1);

	return((long)rl.rlim_max);
}

/*
	Internal. Set a limit. Will never try to set
	a limit greater than the hard limit.
*/
#ifdef __USE_GNU
int	set_limit(__rlimit_resource_t resource, long limit)
#else
int	set_limit(int resource, long limit)
#endif
{
	rlimit	rl;
	int	n;

	n=getrlimit(resource, &rl);
	if (n==-1) return(-1);

	rl.rlim_cur=((unsigned long)limit > (unsigned long)rl.rlim_max ? rl.rlim_max : limit);
	n=setrlimit(resource, &rl);
	return(n);
}

int	set_corelimit(long limit)
{
	return(set_limit(RLIMIT_CORE,limit));
}

void	set_nocore()
{
	set_corelimit(0);
}

void	ignore_stop_signals()
{
#ifdef SIGTTOU
	signal(SIGTTOU, SIG_IGN);
#endif

#ifdef SIGTTIN  /* sent if background process attempts read */
	signal(SIGTTIN, SIG_IGN);  /* ignore signal */
#endif

#ifdef SIGTSTP  /* keyboard-generated stop signal */
	signal(SIGTSTP, SIG_IGN);  /* ignore signal */
#endif
}

void	v_daemonize(int tofork)
{
	if (tofork==1)
	{
		if (fork()!=0) exit(0);
	}
	ignore_stop_signals();
	close_all_fds();
	chdir("/");
	umask(022);	// This should be 0, but i prefer to be secure and incompatible in this case
	disassociate_terminal();
}
