/****************************************/
/* Set the program name (if possible)	*/
/* This way,ps will display a nice name	*/
/****************************************/
/* Start      : 03/12/2001		*/
/* Program ID : 			*/
/* Version    : 1.00			*/
/* End        : 			*/
/* Last Update: 03/12/2001		*/
/* Comments   : 			*/
/*                                      */
/****************************************/

/*
	_V_SET_PROGNAME_OVERWRITE_ARGC
		Overwrite argc[]

	_V_SET_PROGNAME_CHANGE_ARGC0
		Change argc[0]
*/

#include "v/lconfig.h"

#include "v/minmax.h"

#include "progname.h"
#include "string.h"
#include "stdlib.h"

#ifndef NO_NAMESPACES
#	if defined(V_SYS_LINUX) || defined(V_SYS_SOLARIS) || defined(V_SYS_IRIX)
#		define _V_SET_PROGNAME_OVERWRITE_ARGC
#	elif defined(V_SYS_BSD)
#		define _V_SET_PROGNAME_CHANGE_ARGC0
#	endif
#endif

#ifdef HAVE_SETPROCTITLE

#include "sys/types.h"
#include "unistd.h"

int	set_progname(int argn, char *argc[], const char *name)
{
	setproctitle("-%s",name);
}

#elif defined(_V_SET_PROGNAME_OVERWRITE_ARGC)

#define	ARGC_PADDING	0	// the char to use, to fill the unused space

extern char **environ;

namespace VLIB_SET_PROGNAME
{
	int		init=0;		// initialized or not
	static char	*noargument="";	// this static should not hurt

	char		*progname;	// this shows where we are going to place
						// the name
	long		max_size=0;	// the maximum length of the progname
}

using namespace VLIB_SET_PROGNAME;

void	init_progname(int argn, char *argc[])
{
	char	*the_end;	// end of parameters/environment (the last NULL)
	int	n;
	char	**pp;

	progname=argc[0];

	/*
		For all the arguments, check those that are
		continouus and find the end
	*/
	n=0;
	do {
		the_end=argc[n]+strlen(argc[n]);
		n++;
	} while ((n<argn) && (argc[n]==the_end+1));

	while (--n)
	{
		argc[n]=noargument;	// destroy all arguments. They will be overwriten
	}

	n=0;
	while ((environ[n]!=NULL) && (environ[n]==the_end+1))
	{
		the_end=environ[n]+strlen(environ[n]);
		n++;
	}

	for (n=0;environ[n]!=NULL;n++);	// get env size
	n++;	// one for the NULL

	pp=(char **)malloc(n*sizeof(char *));	// allocate mem

	n=0;
	while (environ[n]!=NULL)	// copy environment
	{
		pp[n]=strdup(environ[n]);
		n++;
	}
	pp[n]=NULL;

	environ=pp;
	init=1;

	max_size=(the_end-progname)-1;	// one less for the 0
}

int	set_progname(int argn, char *argc[], const char *name)
{
	int	n,m;

	if (!init) init_progname(argn,argc);

	for (n=0;n<max_size;n++)	// clear the buffer
	{
		progname[n]=ARGC_PADDING;
	}

	m=strlen(name);
	n=MIN(m,max_size);
	memcpy(progname,name,n);
	progname[n]=ARGC_PADDING;

	return(1);
}

#elif defined(_V_SET_PROGNAME_CHANGE_ARGC0)

namespace VLIB_SET_PROGNAME
{
	int	init=0;
}
using namespace VLIB_SET_PROGNAME;

void	init_progname(int argn, char *argc[])
{
	int	n;

	for (n=1;n<argn;n++)
	{
		argc[n]=NULL;
	}

	init=1;
}

int	set_progname(int argn, char *argc[], const char *name)
{
	if (!init)
		init_progname(argn,argc);
	else
		free(argc[0]);	// if it is not the first call
				// free the old data

	argc[0]=strdup(name);

	return(1);
}

#else

int	set_progname(int argn, char *argc[], const char *name)
{
// don't do nothing :)
	return(0);
}

#endif

