/****************************************/
/*					*/
/****************************************/
/* Start      : 09/02/2002		*/
/* Program ID : 			*/
/* Version    : 1.00			*/
/* End        : 			*/
/* Last Update: 09/02/2002		*/
/* Comments   : 			*/
/*                                      */
/****************************************/

#include "sscheduler.h"

#include "v/mem/dlist3.cc"

#include "limits.h"

SSCHEDULER::SSCHEDULER()
{
	next_run=INT_MAX;
}

SSCHEDULER::~SSCHEDULER()
{
}

void SSCHEDULER::Add(time_t t, hook_func func, int flags, int ID)
{
	SCH_ENTRY	s,*p;

	p=get(func,ID);
	if (p!=NULL)
		return;

	s.ID=ID;
	s.func=func;
	s.next_run=time(NULL)+t;
	s.interval=t;
	s.flags=flags;

	hooks.PushL(s);

	if (s.next_run<this->next_run)
		this->next_run=s.next_run;
}

SSCHEDULER::SCH_ENTRY *SSCHEDULER::get(hook_func func, int ID)
{
	SCH_ENTRY	*p;

	hooks.Rewind();

	while ((p=hooks.Get_Next())!=NULL)
	{
		if ((p->func==func) && (p->ID==ID))
		{
			return(p);
		}
	}
	return(NULL);
}

void SSCHEDULER::Cancel(hook_func func, int ID)
{
	SCH_ENTRY	*p;

	p=get(func,ID);
	if (p!=NULL)
		hooks.RemoveItem(p);

	this->next_run=CalcNextRun();
}

time_t SSCHEDULER::Check()
{
	SCH_ENTRY	*p;
	time_t		now=time(NULL);
	hook_func	hf;

	if (now<next_run)
		return(next_run);

	hooks.Rewind();
	while ((p=hooks.Get_Next())!=NULL)
	{
		if (p->next_run<=now)
		{
			hf=p->func;
			if (p->flags & SCH_RESTART)
				p->next_run=now+p->interval;	// Reschedule it from now. not from the scheduled run
			else
			{
				hooks.RemoveCurrent();
				hooks.Rewind();
			}
			(*hf)();
		}
	}
	next_run=CalcNextRun();

	return(next_run);
}

time_t SSCHEDULER::CalcNextRun()
{
	SCH_ENTRY	*p;
	time_t		now=time(NULL);
	time_t		next=INT_MAX;

	hooks.Rewind();
	while ((p=hooks.Get_Next())!=NULL)
	{
		if (p->next_run<=now)	// if this is for the past (or now)
			return(now);

		if (p->next_run<next)
			next=p->next_run;
	}

	return(next);
}

