/****************************************/
/*  VAR class (header)			*/
/****************************************/
/* Start      : 02/07/2001		*/
/* Program ID : 			*/
/* Version    : 1.00			*/
/* End        : 			*/
/* Last Update: 02/07/2001		*/
/* Comments   : 			*/
/*                                      */
/****************************************/

#ifndef _VLIB_V_MISC_VAR_H
#define _VLIB_V_MISC_VAR_H

#include "v/mem/longline.h"
#include "v/mem/vtable.h"
#include "v/misc/keytable2.h"

/*
	VAR class:

	VAR can be a string OR a number AND a table of strings OR numbers
	VAR can also have named child variables of type VAR.

	this way:

	struct A
	{
		int a;
		int b;
		int d[10];
		struct B
		{
			int c;
		} e;
  	} a;

	is:

	VAR a;
	a[a]=1;
	a[b]=2;
	a[d][0]=1;
	a[d][9]=4;
	a[e][c]=3;

	setting a.autocreate=0, will disable the above behavior
*/

/*
	VAR::DVAR:

	It will hold any value of type int or string.
	The last assignment determines the type of the var.
	if it was assigned an integer:
		casting to int will return the integer
		casting to LONGLINE/char * will return the integer as a string
  	if it was assigned a string:
		casting to int will return nothing usefull.. undetermined
		castin to LONGLINE/char * will return the string

  	when a DVAR is constructed, it is assumed to be an integer (0)
*/
/*
	To use the children you can use:
		VAR v;
		v[a]=5;
		v[a][b]=6
	or:
		VAR v,*pv;
		v.AddChild("A");
		pv=&v["A"]; // or pv=cExist("A");
		*pv=5;
		pv->AddChild("B");
		pv=&((*pv)["B"]);
		*pv=6;

  	or a mix of those too..
*/
class VAR
{
 protected:
	int 		Autocreate;	// 0: A[c] will not create c if it doesn't exist
					// 1: A[c] will create c if it doesn't exist
 public:
	class DVAR
	{
	 public:
		enum dvType
		{
			dvtype_int	= 1,
			dvtype_str	= 2,
			dvtype_null	= 3,	// if the type is this, then this is
						// the same as a NULL pointer
						// The object's behavior is undefined in this case
			dvtype_null_fixed = 4	// always NULL. in this case, any assignment will
						// behave like writing to /dev/null
		};
	 protected:
	 	long		iv;	// integer value
		LONGLINE	lv;	// longline value
		dvType		dvtype;
	 protected:
		void	ddd		(DVAR &d);
		inline	int isNULLfixed	();
		void	Construct	();
	 public:
			DVAR		();
			virtual ~DVAR	();
			DVAR		(DVAR &d);
		DVAR	operator=	(DVAR &d);
		DVAR	operator=	(int n);
		DVAR	operator=	(const LONGLINE& ll);
		operator int		();
		operator long		();
		operator char *		();
//		operator LONGLINE	();
		operator const LONGLINE& ();
		dvType	Type		();	// Return the current type
		int	isNULL		();	// Return 1 if it is of type dvtype_null, or else 0
	};
 	enum vType
	{
		vtype_var	= 1,
//		vtype_table	= 2,
		vtype_null	= 3	// To be used by VARNULL. Once it is set, it cannot
					// change the object will always be null (or /dev/null :)
					// Any writes will do nothing. It will always return
					// a DVARNULL instead of DVAR
	};

	vType		vtype;		// The type of the data
	int		isNode;		// If it is a node (there are childs)
	LONGLINE	name;		// The name of the variable (or the node :)
	KEYTABLE<VAR *>	childs;		// the child variables // Keys are converted to uppercase
//	VTABLE<DVAR>	table;
	VTABLE<VAR *>	table;
	DL_List<VAR *>	list;
	int		tsize,tmax;	// number of elements in table and maximum index number

	DVAR		*data;
 protected:
	void	ddd		(VAR &d);
	void	Construct	(const char *name);
	void	settype		(vType v);
	void	ClearTable	();	// Delete the table data..
	void	ClearChilds	();	// Delete all the childs
	void	ClearList	();	// Clear the list
	virtual void	allocdata	();
 public:
		VAR		(const char *name="");
		VAR		(const LONGLINE& ll);
		virtual ~VAR	();
		VAR		(VAR &d);
	VAR	operator=	(VAR &d);

	// casting
	operator int		();
	operator long		();
	operator char *		();
//	operator LONGLINE	();
	operator const LONGLINE& ();
	operator DVAR&		();

	// assignment
	VAR	operator=	(long n);
	VAR	operator=	(int n);
	VAR	operator=	(const LONGLINE& ll);

	// Node stuff
	VAR	*AddChild	(const char *Name);
	VAR	*cExist		(const char *Name);	// Return the child, or NULL
	VAR	&operator[]	(const char *Name);	// Return the child. Create it if it doesn't exist

	// Table stuff
//	DVAR&	operator[]	(long index);	// This will return the wanted table entry
//						// Notes of VTABLE apply here too
	VAR&	operator[]	(long index);	// This will return the wanted table entry
						// Notes of VTABLE apply here too
	VAR&	operator[]	(int index);
	// Misc
	int	isNULL		();		// if vtype==vtype_null return 1 else 0

	void	Dump		(int depth=0, int index=-1);	// Show the tree
	void	Clear		();				// Clear the variable (will not change autocreate)

	void	Rewind		();	// Rewind the list
	VAR	*Get_Next	();	// Get the next one, or NULL
	void	Push		(VAR &v);	// Same as DL_List
	void	PushL		(VAR &v);
	void	Push		(long n);
	void	Push		(int n);
	void	Push		(const LONGLINE& ll);
	void	PushL		(long n);
	void	PushL		(int n);
	void	PushL		(const LONGLINE& ll);
	VAR	Pop		();	// Almost same as DL_List. Return the popped object (not a pointer)
	VAR	PopL		();	// or varnull if there are no more objects
	void	RemoveCurrent	();	// Same as DL_List::RemoveCurrent2() - Remove the current object

	void	autocreate	(int n);// set autocreate ti this and the childs
};

class VARNULL : public VAR
{
 public:
	class DVARNULL : public DVAR
	{
	 public:
		DVARNULL();
		virtual ~DVARNULL();
	};
//	DVARNULL	data;
 private:
 	virtual void allocdata();
 public:
	VARNULL();
	virtual ~VARNULL();
};

#endif

