/****************************************/
/*  SOCKETCONTROL class                 */
/****************************************/
/* Start      : 4/6/98                  */
/* Program ID :                         */
/* Version    : 1.00                    */
/* End        :                         */
/* Last Update: 4/6/98                  */
/* Comments   : Class for controlling   */
/*              sockets (for select)    */
/****************************************/

#include "scontrol.h"
#include "v/mem/dlist3.cc"

SOCKETCONTROL::SOCKETCONTROL()
{
	Next_ID=1L;
	FD_ZERO(&fdr);
	FD_ZERO(&fdw);
	FD_ZERO(&fde);
	MaxWait=_SOCKETCONTROL_MAXWAIT;
	LOOPMODE=_SOCKETCONTROL_LOOPMODE_ONELOOP;
	Maxfd=-1;
}

SOCKETCONTROL::~SOCKETCONTROL()
{
}

void SOCKETCONTROL::Main()
{
	struct timeval	t,*pt;
	int		n;
	SOCKETBASE	**p;
	int		m;
	fd_set		fdr2,fdw2,fde2;

	DONE=0;

	while (!DONE)
	{
		ReSetfdsets();
		memcpy(&fdr2,&fdr,sizeof(fdr));
		memcpy(&fdw2,&fdw,sizeof(fdw));
		memcpy(&fde2,&fde,sizeof(fde));

		t.tv_sec=MaxWait;
		t.tv_usec=0;

		if (MaxWait==0) pt=NULL;
		else pt=&t;

		n=select(Maxfd+1,&fdr2,&fdw2,&fde2,pt);

		socks.Rewind();
		while ((p=socks.Get_Next())!=NULL)
		{
			m=(*p)->Run();
		}
		if (n>0) /* Data */
		{
			SOCKETBASE		***p2; // yoohoo
			DL_List<SOCKETBASE **>	for_dereg;

			socks.Rewind();

			while ((p=socks.Get_Next())!=NULL)
			{
//-     m=(*p)->Run();
    /* Split this to 3. First check for error, then for read, then for write */
    /* This way, there can be 3 handlers for the same socket */
				if (FD_ISSET((*p)->fd,&fde2))
				{
					m=(*p)->RunExcept();
					if (m==_SELECT_DONE) DONE=1;
					if (m==_SELECT_DEREGISTER)
						for_dereg.PushL(p);
				} else
				if (FD_ISSET((*p)->fd,&fdr2))
				{
					m=(*p)->RunRead();
					if (m==_SELECT_DONE) DONE=1;
					if (m==_SELECT_DEREGISTER)
						for_dereg.PushL(p);
				} else
				if (FD_ISSET((*p)->fd,&fdw2))
				{
					if ((*p)->WaitConnect)
						m=(*p)->RunConnect();	// Handle non blocking connect() 2/1/2002
					else
						m=(*p)->RunWrite();
					if (m==_SELECT_DONE) DONE=1;
					if (m==_SELECT_DEREGISTER)
						for_dereg.PushL(p);
				}
			}

			for_dereg.Rewind();
			while ((p2=for_dereg.Get_Next())!=NULL)
			{
   // for_dereg.RemoveCurrent2();
   // for_dereg.Rewind();
//    socks.RemoveItem(FindID((*p)->ID));
				socks.RemoveItem(*p2); //This way, there is no prob if p is already deleted (11/7/2001)
//	Deregister((*p)->ID);
			}

		} /* n==0: timeout    n<0 error */

		if (LOOPMODE==_SOCKETCONTROL_LOOPMODE_ONELOOP) DONE=1;
	}
}

long SOCKETCONTROL::Register(SOCKETBASE *p)
{
	socks.PushL(p);
	p->ID=Next_ID++;
	p->Registered=1;

	Fdset(p);
	Maxfd=MAX(Maxfd,p->fd);

	return(p->ID);
}

SOCKETBASE **SOCKETCONTROL::FindID(long ID)
{
	SOCKETBASE **p,**ret=NULL;
	int OK=0;

	socks.Rewind();
	while (((p=socks.Get_Next())!=NULL) && (!OK))
	{
		if ((*p)->ID==ID) { OK=1; ret=p; }
	}

	return(ret);
}

SOCKETBASE **SOCKETCONTROL::Findfd(int fd)
{
	SOCKETBASE **p;
	int OK=0;

	socks.Rewind();
	while (((p=socks.Get_Next())!=NULL) && (!OK))
	{
		if ((*p)->fd==fd) OK=1;
	}

	return(p);
}

void SOCKETCONTROL::Fdset(SOCKETBASE *p)
{
	if (p->WaitRead)
		FD_SET(p->fd,&fdr);
	else
		FD_CLR(p->fd,&fdr);

	if ((p->WaitWrite) || (p->WaitConnect))
		FD_SET(p->fd,&fdw);
	else
		FD_CLR(p->fd,&fdw);

	if (p->WaitExcept)
		FD_SET(p->fd,&fde);
	else
		FD_CLR(p->fd,&fde);

}

void SOCKETCONTROL::Deregister(long ID)
{
	SOCKETBASE **p;

	p=FindID(ID);
	if (p!=NULL)
	{
		FD_CLR((*p)->fd,&fdr);
		FD_CLR((*p)->fd,&fdw);
		FD_CLR((*p)->fd,&fde);
		socks.RemoveItem(p);
	}

	CalcMaxfd();
}

void SOCKETCONTROL::CalcMaxfd()
{
	int n=0;
	SOCKETBASE **p;

	socks.Rewind();

	while ((p=socks.Get_Next())!=NULL)
	{
		n=MAX((*p)->fd,n);
	}

	Maxfd=n;
}

void SOCKETCONTROL::ReSetfdsets()
{
	SOCKETBASE **p;

	FD_ZERO(&fdr);
	FD_ZERO(&fdw);
	FD_ZERO(&fde);

	socks.Rewind();
	while ((p=socks.Get_Next())!=NULL)
	{
		Fdset(*p);
	}

	CalcMaxfd();
}

