/****************************************/
/*  SOCKET class (header)               */
/****************************************/
/* Start      : 11/10/98                */
/* Program ID :                         */
/* Version    : 1.10                    */
/* End        :                         */
/* Last Update:                         */
/* Comments   : 12/7/2001               */
/*                                      */
/****************************************/

#ifndef _SOCKET2_HPP
#define _SOCKET2_HPP

#error Depricated -- use socket3

#include "sys/types.h"
#include "sys/socket.h"
#include "sys/un.h"
#include "netinet/in.h"
#include "stdio.h"
#include "errno.h"
#include "unistd.h"
#include "arpa/inet.h"
#include "fcntl.h"
#include "stdlib.h"
#include "string.h"

/*
 SRV:  socket->bind->listen->accept
 CLNT: socket->connect
 */

/*
	26/11/2001:	v1.10:
		Added unix domain sockets

	12/7/2001:	v1.00:

		SOCKET_* use the handle_error (default 1).
		handle_error == 0 : Don't do nothing. Return the return code of the called function
		handle_error == 1 : Display some info and exit
*/

/*
 SOCKET class

 */

#define _SOCKET_ERROR_SOCKET  1
#define _SOCKET_ERROR_BIND    2
#define _SOCKET_ERROR_LISTEN  3
#define _SOCKET_ERROR_ACCEPT  4
#define _SOCKET_ERROR_CONNECT 5
#define _SOCKET_ERROR_READ    6
#define _SOCKET_ERROR_WRITE   7
#define _SOCKET_ERROR_CLOSE   8
#define _SOCKET_MAX_QUEUE  1  //"very" max is 5


#define SOCK_SOCKET  1
#define SOCK_BIND    2
#define SOCK_LISTEN  4
#define SOCK_ACCEPT  8
#define SOCK_CONNECT 16

class SOCKET
{
 private:
 public:
  long status;
  int sockfd;
  int domain, type, protocol;

  struct sockaddr_in sin;  /* the port is ntohs(sin.sin_port) */
//  int sin_size;

  struct sockaddr_in remote;
//  int remote_size;

  struct sockaddr_un ssun;
//  int sun_size;

 private:
  void vvv(const SOCKET& s);
 public:
  SOCKET();
  ~SOCKET();
  SOCKET(const SOCKET &s);
  SOCKET &operator=(const SOCKET &s);
  void Show();
};


class SOCKET_COMMON : public SOCKET
{
 private:
 public:
  int socket_auto;  /* auto - socket, listen*/
  int handle_errors;	// Handle errors. Display some info and then exit..
 private:
  void vvv(const SOCKET_COMMON& s);
 public:
  SOCKET_COMMON();
  ~SOCKET_COMMON();
  SOCKET_COMMON(const SOCKET_COMMON &s);
  SOCKET_COMMON(const SOCKET &s);
  SOCKET_COMMON &operator=(const SOCKET_COMMON &s);
  int Socket(int domain1,int type1,int protocol1);
  void Socket();
  int Read(int s1,char *buf1,int buflen1);
  int Read(char *buf1,int buflen1);
  int Write(int s1,char *buf1,int buflen1);
  int Write(char *buf1,int buflen1);
  int Close(int s1);
  int Close();
  void ERROR(int n);
  void SetNonBlock();
  void SetBlock();

};

/*
 socket -> bind

 Connect(address, port); // address,port is Host-ordered
 */
class SOCKET_CONNECT : public SOCKET_COMMON
{
 private:
 public:
 private:
 public:
  SOCKET_CONNECT();
  ~SOCKET_CONNECT();
  int Connect(int s1, struct sockaddr *name, int namelen);
//  SOCKET Connect(long address, int port1);
   SOCKET *Connect(long address, short int port1);
};

/*
 socket -> bind -> listen -> accept

 Bind(port);  // port is Host-ordered
 Accept();
 */
class SOCKET_LISTEN : public SOCKET_COMMON
{
 private:
 public:
 private:
 public:
  SOCKET_LISTEN();
  ~SOCKET_LISTEN();
  int Bind(int s1, struct sockaddr *name1, int namelen);
  int Bind(short int port);
  int Listen(int s1, int backlog);
  int Listen();
  int Accept(int s1,struct sockaddr *addr,socklen_t *addrlen);
  SOCKET Accept();
};

class SOCKET_CONNECT_UNIX : public SOCKET_CONNECT
{
 public:
	SOCKET_CONNECT_UNIX();
	~SOCKET_CONNECT_UNIX();
	SOCKET *Connect(const char *path);
};

class SOCKET_LISTEN_UNIX : public SOCKET_LISTEN
{
 public:
 	SOCKET_LISTEN_UNIX();
	~SOCKET_LISTEN_UNIX();
	int Bind(const char *path);
};

#endif


/*
 struct sockaddr_in {
  short int             sin_family;     // Address family
  unsigned short int    sin_port;       // Port number
  struct in_addr        sin_addr;       // Internet address

  // Pad to size of `struct sockaddr'.
  unsigned char         __pad[__SOCK_SIZE__ - sizeof(short int) -
                        sizeof(unsigned short int) - sizeof(struct in_addr)];
  };

  struct in_addr {
          __u32   s_addr;
  };

*/
