/****************************************/
/*  SOCKET class                        */
/****************************************/
/* Start      : 11/10/98                */
/* Program ID :                         */
/* Version    : 2.01                    */
/* End        :                         */
/* Last Update:                         */
/* Comments   : 12/04/2004              */
/*                                      */
/****************************************/

/*
 	12/ Apr 2004	v2.01:	Added shutdown()
	1/2/2002	v2.00:	Almost-rewritten
				Changed classes
				Incompatible with older version
*/

#include "v/lconfig.h"
#include "socket3.h"

#include "v/file/fileops.h"

#include "stdio.h"
#include "errno.h"
#include "unistd.h"
#include "fcntl.h"
#include "stdlib.h"
#include "string.h"

#ifdef BZERO_IN_STRINGS_H
#	include "strings.h"
#endif

#ifndef UNIX_PATH_MAX
#	define 	UNIX_PATH_MAX	108
#endif

SOCKET::SOCKET()
{
//	status=0;
	sockfd=-1;
	last_error=0;
}

SOCKET::~SOCKET()
{
}

void SOCKET::Show()
{
	printf(" {sockfd,domain,type,protocol}={%d,%d,%d,%d}\n",
		sockfd,domain,type,protocol);
	if (domain == PF_INET)
	{
		printf(" {sin.sin_family,sin.sin_port,sin.sin_addr}={%d,%d,%x}\n",
			addr.sin.sin_family,
			(int)ntohs((short)addr.sin.sin_port),
			ntohl(addr.sin.sin_addr.s_addr));
		printf(" {remote.sin_family,remote.sin_port,remote.sin_addr}={%d,%d,%x}\n",
			remote.sin.sin_family,
			(int)ntohs((short)remote.sin.sin_port),
			ntohl(remote.sin.sin_addr.s_addr));
	} else
	{
		printf(" {ssun.sun_family,ssun.sun_path}={%d,%s}\n",
			addr.ssun.sun_family,
			addr.ssun.sun_path);
	}
}


SOCKET::SOCKET(const SOCKET &s)
{
	vvv(s);
}

void SOCKET::vvv(const SOCKET& s)
{
//	status=s.status;
	sockfd=s.sockfd;
	domain=s.domain;
	type=s.type;
	protocol=s.protocol;

	memcpy(&addr,&s.addr,sizeof(addr));
	memcpy(&remote,&s.remote,sizeof(remote));
}

SOCKET &SOCKET::operator=(const SOCKET &s)
{
	vvv(s);

	return(*this);
}

/********** SOCKET_COMMON *************/

SOCKET_COMMON::SOCKET_COMMON()
{
	init();
}

SOCKET_COMMON::~SOCKET_COMMON()
{
}

void SOCKET_COMMON::init()
{
	socket_auto=1;
	handle_errors=1;
	ready_to_accept=0;
}

void SOCKET_COMMON::vvv(const SOCKET_COMMON& s)
{
	socket_auto=s.socket_auto;
	handle_errors=s.handle_errors;
	ready_to_accept=s.ready_to_accept;
}

SOCKET_COMMON::SOCKET_COMMON(const SOCKET_COMMON &s) : SOCKET(s)
{
	vvv(s);
}

SOCKET_COMMON::SOCKET_COMMON(const SOCKET &s) : SOCKET(s)
{
	init();
}


SOCKET_COMMON &SOCKET_COMMON::operator=(const SOCKET_COMMON &s)
{
	SOCKET::operator=(s);
	vvv(s);

	return(*this);
}

SOCKET_COMMON &SOCKET_COMMON::operator=(const SOCKET &s)
{
	SOCKET::operator=(s);
	init();

	return(*this);
}

void SOCKET_COMMON::ERROR(int n)
{
	char msg[100];

	last_error=errno;
	if ((!handle_errors) || (!n))
		return;

	switch (n)
	{
		case _SOCKET_ERROR_SOCKET:
			sprintf(msg,"Error creating socket");
			break;
		case _SOCKET_ERROR_BIND:
			sprintf(msg,"Error binding socket");
			break;
		case _SOCKET_ERROR_LISTEN:
			sprintf(msg,"Error listening to socket");
			break;
		case _SOCKET_ERROR_ACCEPT:
			sprintf(msg,"Error accepting connections");
			break;
		case _SOCKET_ERROR_CONNECT:
			sprintf(msg,"Error creating connection");
			break;
		case _SOCKET_ERROR_READ:
			sprintf(msg,"Error reading from socket");
			break;
		case _SOCKET_ERROR_WRITE:
			sprintf(msg,"Error writing to socket");
			break;
		case _SOCKET_ERROR_CLOSE:
			sprintf(msg,"Error closing socket");
			break;
		case _SOCKET_ERROR_SETSOCKOPT:
			sprintf(msg,"Error with setsockopt()");
			break;
		case _SOCKET_ERROR_GETSOCKOPT:
			sprintf(msg,"Error with getsockopt()");
			break;
		case _SOCKET_ERROR_SOCKFD:
			sprintf(msg,"sockfd==-1");
			break;
		case _SOCKET_ERROR_SHUTDOWN:
			sprintf(msg,"Error with shutdown()");
			break;
	};
	printf(" class SOCKET error No %d\n\r",n);
	printf(" %s \n\r",msg);
	printf(" errno = %d\n\r",errno);
	perror("PERROR()");
	printf(" {s,domain,type,protocol}={%d,%d,%d,%d}\n\r",sockfd,domain,type,protocol);
	printf(" {port}={%d}\n\r",(int)(ntohs(addr.sin.sin_port)));
	exit(-1*n);
}

int SOCKET_COMMON::Close()
{
	int ret=0;

	if (sockfd==-1)
		return(0);

	ret=close(sockfd);
	sockfd=-1;
	if (ret==-1)
		ERROR(_SOCKET_ERROR_CLOSE);

	return(ret);
}

int SOCKET_COMMON::Shutdown()
{
	int ret=0;

	if (sockfd==-1)
		return(0);

	ret=shutdown(sockfd, SHUT_RDWR);
	if (ret==-1)
		ERROR(_SOCKET_ERROR_SHUTDOWN);

	Close();

	return(ret);
}

ssize_t SOCKET_COMMON::Read(char *buf,int buflen)
{
	ssize_t	ret=0;

	if (sockfd==-1)
		return(0);

	ret=read(sockfd,buf,buflen);
	if (ret==-1)
		ERROR(_SOCKET_ERROR_READ);

	return(ret);
}

ssize_t SOCKET_COMMON::Write(char *buf,int buflen)
{
	ssize_t	ret=0;

	if (sockfd==-1)
		return(0);

	ret=write(sockfd,buf,buflen);
	if (ret==-1)
		ERROR(_SOCKET_ERROR_READ);

	return(ret);
}

void SOCKET_COMMON::SetNonBlock()
{
	if (sockfd==-1)
		return;

	set_block(sockfd,0);
//	fcntl(sockfd,F_SETFL,O_NONBLOCK);
}

void SOCKET_COMMON::SetBlock()
{
	if (sockfd==-1)
		return;

	set_block(sockfd,0);
//	fcntl(sockfd,F_SETFL,0);
}

void SOCKET_COMMON::SetLinger(int onoff, int timeout)
{
	int		ret;
	struct linger	lin;

	if (sockfd==-1)
		return;

	lin.l_onoff=onoff;
	lin.l_linger=timeout;

	ret=setsockopt(sockfd,SOL_SOCKET,SO_LINGER,&lin,sizeof(lin));

	if (ret==-1)
		ERROR(_SOCKET_ERROR_SETSOCKOPT);
}

/*
	Check if there is an error on a socket.
	Return: 1: Yes
		0: No
*/
int SOCKET_COMMON::HasError()
{
	int		ret,n;
	socklen_t	m;

	if (sockfd==-1)
		return(1);

	m=sizeof(n);
	ret=getsockopt(sockfd,SOL_SOCKET,SO_ERROR,&n,&m);
	if (ret==-1)
	{
		ERROR(_SOCKET_ERROR_GETSOCKOPT);
		n=1;
	}

	return(n);
}

int	SOCKET_COMMON::Socket(int domain, int type, int protocol)
{
	this->domain=domain;
	this->type=type;
	this->protocol=protocol;

	sockfd=socket(domain,type,protocol);
	if (sockfd==-1)
		ERROR(_SOCKET_ERROR_SOCKET);

	return(sockfd);
}

int	SOCKET_COMMON::Listen(int backlog)
{
	int	ret;

	ready_to_accept=0;

	if (sockfd==-1) /* We cannot auto-bind (where?) */
	{
		ERROR(_SOCKET_ERROR_SOCKFD);
		return(-1);
	}

	ret=listen(sockfd,backlog);
	if (ret==-1)
		ERROR(_SOCKET_ERROR_LISTEN);
	else
		ready_to_accept=1;

	return(ret);
}

SOCKET	SOCKET_COMMON::Accept()
{
	SOCKET		sock;
	int		ret;
	socklen_t	slen;

	if ((!ready_to_accept) && (socket_auto))
		Listen();

	if (!ready_to_accept)	// suppose that accept() will fail
		return(sock);

	slen=sizeof(sock.remote);
	ret=accept(sockfd,(struct sockaddr *)&sock.remote,&slen);	// TODO: split this to TCPUDP & Unix
	if ((ret==-1) && (errno!=EAGAIN) && (errno!=EWOULDBLOCK))
		ERROR(_SOCKET_ERROR_ACCEPT);

	sock.sockfd=ret;
	memcpy(&sock.addr,&addr,sizeof(addr));
	sock.domain=domain;
	sock.type=type;
	sock.protocol=protocol;

	ready_to_accept=0;

	return(sock);
}

/********** SOCKET_COMMON_TCPUDP **********/

SOCKET_COMMON_TCPUDP::SOCKET_COMMON_TCPUDP()
{
}

SOCKET_COMMON_TCPUDP::~SOCKET_COMMON_TCPUDP()
{
}

/*
	address, port NBO
*/
int	SOCKET_COMMON_TCPUDP::Connect(int address, short int port)
{
	int	ret;

	if (sockfd==-1)
	{
		if (socket_auto)
			Socket();

		if (sockfd==-1)
		{
			ERROR(_SOCKET_ERROR_SOCKFD);
			return(-1);
		}
	}

	remote.sin.sin_family=AF_INET;
	remote.sin.sin_port=port;
	remote.sin.sin_addr.s_addr=address;

	ret=connect(sockfd,(struct sockaddr *)&remote.sin,sizeof(remote.sin));

	if ((ret==-1) && (errno!=EINPROGRESS))
		ERROR(_SOCKET_ERROR_CONNECT);

	return(ret);
}

/*
	The address and the port to bind to
*/
int	SOCKET_COMMON_TCPUDP::Bind(int address, short int port) // NBO
{
	int	ret;

	if (sockfd==-1)
	{
		if (socket_auto)
			Socket();

		if (sockfd==-1)
		{
			ERROR(_SOCKET_ERROR_SOCKFD);
			return(-1);
		}
	}

	addr.sin.sin_family=AF_INET;
	addr.sin.sin_port=port;
	addr.sin.sin_addr.s_addr=address;

	ret=bind(sockfd,(struct sockaddr *)&addr.sin,sizeof(addr.sin));
	if (ret==-1)
		ERROR(_SOCKET_ERROR_BIND);

	return(ret);
}

int	SOCKET_COMMON_TCPUDP::Bind(short int port) // HBO
{
	return(Bind(INADDR_ANY,htons(port)));
}

/************** SOCKET_TCP ****************/

SOCKET_TCP::SOCKET_TCP()
{
}

SOCKET_TCP::~SOCKET_TCP()
{
}

int	SOCKET_TCP::Socket()
{
	SOCKET_COMMON::Socket(PF_INET,SOCK_STREAM,0);

	return(sockfd);
}

/************** SOCKET_UDP ****************/

SOCKET_UDP::SOCKET_UDP()
{
}

SOCKET_UDP::~SOCKET_UDP()
{
}

int	SOCKET_UDP::Socket()
{
	SOCKET_COMMON::Socket(PF_INET,SOCK_DGRAM,0);

	return(sockfd);
}

/************** SOCKET_UNIX ****************/

SOCKET_UNIX::SOCKET_UNIX()
{
}

SOCKET_UNIX::~SOCKET_UNIX()
{
}

int	SOCKET_UNIX::Socket()
{
	SOCKET_COMMON::Socket(PF_UNIX,SOCK_STREAM,0);

	return(sockfd);
}

int	SOCKET_UNIX::Bind(const char *path)
{
	int	ret;

	if (sockfd==-1)
	{
		if (socket_auto)
			Socket();

		if (sockfd==-1)
		{
			ERROR(_SOCKET_ERROR_SOCKFD);
			return(-1);
		}
	}

	addr.ssun.sun_family=AF_UNIX;
	strncpy(addr.ssun.sun_path,path,UNIX_PATH_MAX);

	ret=bind(sockfd,(struct sockaddr *)&addr.ssun,sizeof(addr.ssun));
	if (ret==-1)
		ERROR(_SOCKET_ERROR_BIND);

	return(ret);
}

int	SOCKET_UNIX::Connect(const char *path)
{
	int	ret;

	if (sockfd==-1)
	{
		if (socket_auto)
			Socket();

		if (sockfd==-1)
		{
			ERROR(_SOCKET_ERROR_SOCKFD);
			return(-1);
		}
	}

	remote.ssun.sun_family=AF_UNIX;
	strncpy(remote.ssun.sun_path,path,UNIX_PATH_MAX);

	ret=connect(sockfd,(struct sockaddr *)&remote.ssun,sizeof(remote.ssun));

	if (ret==-1)
		ERROR(_SOCKET_ERROR_CONNECT);

	return(ret);
}


